=== modified file 'INSTALL'
--- INSTALL	2010-01-29 10:36:23 +0000
+++ INSTALL	2010-06-02 05:03:31 +0000
@@ -15,10 +15,10 @@
 
 bzr can optionally use compiled versions of some parts of the code
 for increased speed. When installing bzr you need the ability to
-build C extensions. Some Linux distributions package the necessary
+build C extensions. Some GNU/Linux distributions package the necessary
 headers separately from the main Python package. This package is
 probably named something like python-dev or python-devel. FreeBSD,
-Windows, source-based Linux distributions, and possibly other operating
+Windows, source-based GNU/Linux distributions, and possibly other operating
 systems, have the required files installed by default.
 
 If you are installing bzr from a bzr branch rather than a release tarball,

=== modified file 'NEWS'
--- NEWS	2010-05-31 20:21:04 +0000
+++ NEWS	2010-06-17 09:11:03 +0000
@@ -14,23 +14,81 @@
 Compatibility Breaks
 ********************
 
+* URLs like ``foo:bar/baz`` are now always parsed as a URL with scheme "foo"
+  and path "bar/baz", even if bzr does not recognize "foo" as a known URL
+  scheme.  Previously these URLs would be treated as local paths.
+  (Gordon Tyler)
+
 New Features
 ************
 
+* Support ``--directory`` option for a number of additional commands:
+  conflicts, merge-directive, missing, resolve, shelve, switch,
+  unshelve, whoami. (Martin von Gagern, #527878)
+
 Bug Fixes
 *********
 
+* ``bzr init`` does not recursively scan directory contents anymore
+  leading to faster init for directories with existing content.
+  (Martin [gz], Parth Malwankar, #501307)
+
+* ``bzr log --exclude-common-ancestry`` is now taken into account for
+  linear ancetries. (Vincent Ladeuil, #575631)
+
+* ``bzr pull`` now works when a lp: URL is explicitly defined as the parent
+  or pull location in locations.conf or branch.conf.
+  (Gordon Tyler, #534787)
+
+* Ensure that wrong path specifications in ``BZR_PLUGINS_AT`` display
+  proper error messages. (Vincent Ladeuil, #591215)
+
+* Explicitly removing ``--profile-imports`` option from parsed command-line
+  arguments on Windows, because bzr script does the same.
+  (Alexander Belchenko, #588277)
+
+* Fetching was slightly confused about the best code to use and was
+  using a new code path for all branches, resulting in more lookups than
+  necessary on old branches. (Robert Collins, #593515)
+
 * Final fix for 'no help for command' issue. We now show a clean message
   when a command has no help, document how to set help more clearly, and
   test that all commands available to the test suite have help.
   (Robert Collins, #177500)
 
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
+* Relative imports in plugins are now handled correctly when using
+  BZR_PLUGINS_AT. (Vincent Ladeuil, #588959)
+
+* ``ScriptRunner`` now strips off leading indentation from test scripts,
+  which previously caused "SyntaxError: No command for line".
+  (Martin Pool)
+
 Improvements
 ************
 
+* Bazaar now reads data from SSH connections more efficiently on platforms
+  that provide the ``socketpair`` function, and when using paramiko.
+  (Andrew Bennetts, #590637)
+
+* ``Branch.copy_content_into`` is now a convenience method dispatching to
+  a ``InterBranch`` multi-method. This permits ``bzr-loom`` and other
+  plugins to intercept this even when a ``RemoteBranch`` proxy is in use.
+  (Robert Collins, #201613)
+
+* Use lazy imports in ``bzrlib/merge.py`` so that plugins like ``news_merge``
+  do not cause modules to be loaded unnecessarily just because the plugin
+  registers a merge hook.  This improves ``bzr rocks`` time by about 25%
+  in a default installation (with just the core plugins).
+  (Andrew Bennetts)
+
 Documentation
 *************
 
+* Added ``regression`` tag to our tags list. (Robert Collins)
+
 * Improved our release checklist to have a bit less churn and leave things
   ready-to-go for the next action (including other people doing
   development). (Robert Collins)
@@ -38,9 +96,30 @@
 API Changes
 ***********
 
+* ``bzrlib.branch.InterBranch._get_branch_formats_to_test`` now returns
+  an iterable of format pairs, rather than just a single pair, permitting
+  InterBranch objects that work with multiple permutations to be
+  comprehensively tested. (Robert Collins)
+
+* ``bzrlib.knit.KnitSequenceMatcher``, which has been deprecated since
+  2007, has been deleted.  Use ``PatienceSequenceMatcher`` from
+  ``bzrlib.patiencediff`` instead. (Andrew Bennetts)
+
+* ``bzrlib.transport.ssh.SSHVendor.connect_ssh`` now returns an object
+  that implements the interface of ``bzrlib.transport.ssh.SSHConnection``.
+  Third-party implementations of ``SSHVendor`` may need to be updated
+  accordingly.  Similarly, any code using ``SSHConnection`` directly will
+  need to be updated.  (Andrew Bennetts)
+
+* The constructor of ``bzrilb.smart.medium.SmartSSHClientMedium`` has
+  changed to take an ``SSHParams`` instance (replacing many individual
+  values).  (Andrew Bennetts)
+
 Internals
 *********
 
+* Improved ``bzrlib.urlutils`` to handle lp:foo/bar URLs. (Gordon Tyler)
+
 Testing
 *******
 
@@ -445,10 +524,28 @@
   Command is now transient and only exists for the duration of ``run()``.
   (Robert Collins)
 
-bzr 2.2.0b1
-###########
-
-:2.2.0b1: 2010-04-01
+bzr 2.2b1
+#########
+
+:2.2b1: 2010-04-01
+
+This is the first beta of the 2.2 series, leading up to a 2.2.0
+release in July or August.  Beta releases are suitable for everyday use
+but may cause some incompatibilities with plugins.  Some plugins may need
+small updates to work with 2.2b1.
+
+2.2b1 includes some changes to make merge conflicts easier to understand
+and resolve.  It also removes some old unnecessary code, and loads
+somewhat less code at startup.  It starts adding a common infrastructure
+for dealing with colocated named branches, which can be implemented in
+various ways in either bzr native or foreign formats.   On Ubuntu and
+other platforms with the apport bug-reporting library, there's an easier
+path to report problems with bzr.  We plan to continue with these themes
+through the 2.2 series.
+
+Over thirty bugs have been fixed, including in the log command, exporting
+to tarballs, restarting interrupted system calls, portability of compiled
+extensions, making backups during upgrade, and locking on ftp.
 
 Compatibility Breaks
 ********************
@@ -730,6 +827,39 @@
   happens, and another warning will be written if the log file could not
   be closed after retrying 100 times.  (Andrew Bennetts, #531746)
 
+bzr 2.1.3
+#########
+
+:Codename: Do run run
+:2.1.3: NOT RELEASED YET
+
+Compatibility Breaks
+********************
+
+New Features
+************
+
+Bug Fixes
+*********
+
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
+Improvements
+************
+
+Documentation
+*************
+
+API Changes
+***********
+
+Internals
+*********
+
+Testing
+*******
+
 bzr 2.1.2
 #########
 
@@ -1225,6 +1355,9 @@
   permissions as ``.bzr`` directory on a POSIX OS.
   (Parth Malwankar, #262450)
 
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
 * Reduce peak memory by one copy of compressed text.
   (John Arbash Meinel, #566940)
 
@@ -3944,8 +4077,8 @@
   can have a large effect on ``bzr checkout`` times. (John Arbash Meinel)
 
 * selftest now supports a --parallel option, with values of 'fork' or
-  'subprocess' to run the test suite in parallel. Currently only linux
-  machine work, other platforms need patches submitted. (Robert Collins,
+  'subprocess' to run the test suite in parallel. Currently only Linux
+  machines work, other platforms need patches submitted. (Robert Collins,
   Vincent Ladeuil)
 
 * ``tests.run_suite`` has a new parameter ``suite_decorators``, a list of 
@@ -6699,7 +6832,7 @@
 
 * bzr main script cannot be imported (Benjamin Peterson)
 
-* On Linux bzr additionally looks for plugins in arch-independent site
+* On GNU/Linux bzr additionally looks for plugins in arch-independent site
   directory. (Toshio Kuratomi)
 
 * The ``set_rh`` branch hook is now deprecated. Please migrate
@@ -7042,7 +7175,7 @@
 
 * BZR_LOG environment variable controls location of .bzr.log trace file.
   User can suppress writing messages to .bzr.log by using '/dev/null'
-  filename (on Linux) or 'NUL' (on Windows). If BZR_LOG variable
+  filename (on Unix) or 'NUL' (on Windows). If BZR_LOG variable
   is not defined but BZR_HOME is defined then default location
   for .bzr.log trace file is ``$BZR_HOME/.bzr.log``.
   (Alexander Belchenko, #106117)

=== modified file 'bzrlib/_annotator_py.py'
--- bzrlib/_annotator_py.py	2009-07-08 23:10:47 +0000
+++ bzrlib/_annotator_py.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,13 +18,15 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from bzrlib import annotate # Must be lazy to avoid circular importing
+from bzrlib import (
+    annotate, # Must be lazy to avoid circular importing
+    graph as _mod_graph,
+    patiencediff,
+    )
 """)
 from bzrlib import (
     errors,
-    graph as _mod_graph,
     osutils,
-    patiencediff,
     ui,
     )
 

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2009-09-14 01:48:28 +0000
+++ bzrlib/annotate.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,11 +28,16 @@
 import sys
 import time
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import (
+    patiencediff,
+    tsort,
+    )
+""")
 from bzrlib import (
     errors,
     osutils,
-    patiencediff,
-    tsort,
     )
 from bzrlib.config import extract_email_address
 from bzrlib.repository import _strip_NULL_ghosts
@@ -312,8 +317,7 @@
 
 
 def _get_matching_blocks(old, new):
-    matcher = patiencediff.PatienceSequenceMatcher(None,
-        old, new)
+    matcher = patiencediff.PatienceSequenceMatcher(None, old, new)
     return matcher.get_matching_blocks()
 
 

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2010-05-25 17:27:52 +0000
+++ bzrlib/branch.py	2010-06-17 06:30:22 +0000
@@ -29,6 +29,7 @@
         errors,
         lockdir,
         lockable_files,
+        remote,
         repository,
         revision as _mod_revision,
         rio,
@@ -197,6 +198,13 @@
         return self.supports_tags() and self.tags.get_tag_dict()
 
     def get_config(self):
+        """Get a bzrlib.config.BranchConfig for this Branch.
+
+        This can then be used to get and set configuration options for the
+        branch.
+
+        :return: A bzrlib.config.BranchConfig.
+        """
         return BranchConfig(self)
 
     def _get_config(self):
@@ -961,7 +969,6 @@
                 raise errors.NoSuchRevision(self, stop_revision)
         return other_history[self_len:stop_revision]
 
-    @needs_write_lock
     def update_revisions(self, other, stop_revision=None, overwrite=False,
                          graph=None):
         """Pull in new perfect-fit revisions.
@@ -1016,7 +1023,6 @@
             self._extend_partial_history(distance_from_last)
         return self._partial_revision_history_cache[distance_from_last]
 
-    @needs_write_lock
     def pull(self, source, overwrite=False, stop_revision=None,
              possible_transports=None, *args, **kwargs):
         """Mirror source into this branch.
@@ -1265,24 +1271,14 @@
                 revno = 1
         destination.set_last_revision_info(revno, revision_id)
 
-    @needs_read_lock
     def copy_content_into(self, destination, revision_id=None):
         """Copy the content of self into destination.
 
         revision_id: if not None, the revision history in the new branch will
                      be truncated to end with revision_id.
         """
-        self.update_references(destination)
-        self._synchronize_history(destination, revision_id)
-        try:
-            parent = self.get_parent()
-        except errors.InaccessibleParent, e:
-            mutter('parent was not accessible to copy: %s', e)
-        else:
-            if parent:
-                destination.set_parent(parent)
-        if self._push_should_merge_tags():
-            self.tags.merge_to(destination.tags)
+        return InterBranch.get(self, destination).copy_content_into(
+            revision_id=revision_id)
 
     def update_references(self, target):
         if not getattr(self._format, 'supports_reference_locations', False):
@@ -3188,11 +3184,17 @@
     _optimisers = []
     """The available optimised InterBranch types."""
 
-    @staticmethod
-    def _get_branch_formats_to_test():
-        """Return a tuple with the Branch formats to use when testing."""
-        raise NotImplementedError(InterBranch._get_branch_formats_to_test)
+    @classmethod
+    def _get_branch_formats_to_test(klass):
+        """Return an iterable of format tuples for testing.
+        
+        :return: An iterable of (from_format, to_format) to use when testing
+            this InterBranch class. Each InterBranch class should define this
+            method itself.
+        """
+        raise NotImplementedError(klass._get_branch_formats_to_test)
 
+    @needs_write_lock
     def pull(self, overwrite=False, stop_revision=None,
              possible_transports=None, local=False):
         """Mirror source into target branch.
@@ -3203,6 +3205,7 @@
         """
         raise NotImplementedError(self.pull)
 
+    @needs_write_lock
     def update_revisions(self, stop_revision=None, overwrite=False,
                          graph=None):
         """Pull in new perfect-fit revisions.
@@ -3216,6 +3219,7 @@
         """
         raise NotImplementedError(self.update_revisions)
 
+    @needs_write_lock
     def push(self, overwrite=False, stop_revision=None,
              _override_hook_source_branch=None):
         """Mirror the source branch into the target branch.
@@ -3226,110 +3230,113 @@
 
 
 class GenericInterBranch(InterBranch):
-    """InterBranch implementation that uses public Branch functions.
-    """
-
-    @staticmethod
-    def _get_branch_formats_to_test():
-        return BranchFormat._default_format, BranchFormat._default_format
-
+    """InterBranch implementation that uses public Branch functions."""
+
+    @classmethod
+    def is_compatible(klass, source, target):
+        # GenericBranch uses the public API, so always compatible
+        return True
+
+    @classmethod
+    def _get_branch_formats_to_test(klass):
+        return [(BranchFormat._default_format, BranchFormat._default_format)]
+
+    @classmethod
+    def unwrap_format(klass, format):
+        if isinstance(format, remote.RemoteBranchFormat):
+            format._ensure_real()
+            return format._custom_format
+        return format                                                                                                  
+
+    @needs_write_lock
+    def copy_content_into(self, revision_id=None):
+        """Copy the content of source into target
+
+        revision_id: if not None, the revision history in the new branch will
+                     be truncated to end with revision_id.
+        """
+        self.source.update_references(self.target)
+        self.source._synchronize_history(self.target, revision_id)
+        try:
+            parent = self.source.get_parent()
+        except errors.InaccessibleParent, e:
+            mutter('parent was not accessible to copy: %s', e)
+        else:
+            if parent:
+                self.target.set_parent(parent)
+        if self.source._push_should_merge_tags():
+            self.source.tags.merge_to(self.target.tags)
+
+    @needs_write_lock
     def update_revisions(self, stop_revision=None, overwrite=False,
         graph=None):
         """See InterBranch.update_revisions()."""
-        self.source.lock_read()
-        try:
-            other_revno, other_last_revision = self.source.last_revision_info()
-            stop_revno = None # unknown
-            if stop_revision is None:
-                stop_revision = other_last_revision
-                if _mod_revision.is_null(stop_revision):
-                    # if there are no commits, we're done.
-                    return
-                stop_revno = other_revno
-
-            # what's the current last revision, before we fetch [and change it
-            # possibly]
-            last_rev = _mod_revision.ensure_null(self.target.last_revision())
-            # we fetch here so that we don't process data twice in the common
-            # case of having something to pull, and so that the check for
-            # already merged can operate on the just fetched graph, which will
-            # be cached in memory.
-            self.target.fetch(self.source, stop_revision)
-            # Check to see if one is an ancestor of the other
-            if not overwrite:
-                if graph is None:
-                    graph = self.target.repository.get_graph()
-                if self.target._check_if_descendant_or_diverged(
-                        stop_revision, last_rev, graph, self.source):
-                    # stop_revision is a descendant of last_rev, but we aren't
-                    # overwriting, so we're done.
-                    return
-            if stop_revno is None:
-                if graph is None:
-                    graph = self.target.repository.get_graph()
-                this_revno, this_last_revision = \
-                        self.target.last_revision_info()
-                stop_revno = graph.find_distance_to_null(stop_revision,
-                                [(other_last_revision, other_revno),
-                                 (this_last_revision, this_revno)])
-            self.target.set_last_revision_info(stop_revno, stop_revision)
-        finally:
-            self.source.unlock()
-
+        other_revno, other_last_revision = self.source.last_revision_info()
+        stop_revno = None # unknown
+        if stop_revision is None:
+            stop_revision = other_last_revision
+            if _mod_revision.is_null(stop_revision):
+                # if there are no commits, we're done.
+                return
+            stop_revno = other_revno
+
+        # what's the current last revision, before we fetch [and change it
+        # possibly]
+        last_rev = _mod_revision.ensure_null(self.target.last_revision())
+        # we fetch here so that we don't process data twice in the common
+        # case of having something to pull, and so that the check for
+        # already merged can operate on the just fetched graph, which will
+        # be cached in memory.
+        self.target.fetch(self.source, stop_revision)
+        # Check to see if one is an ancestor of the other
+        if not overwrite:
+            if graph is None:
+                graph = self.target.repository.get_graph()
+            if self.target._check_if_descendant_or_diverged(
+                    stop_revision, last_rev, graph, self.source):
+                # stop_revision is a descendant of last_rev, but we aren't
+                # overwriting, so we're done.
+                return
+        if stop_revno is None:
+            if graph is None:
+                graph = self.target.repository.get_graph()
+            this_revno, this_last_revision = \
+                    self.target.last_revision_info()
+            stop_revno = graph.find_distance_to_null(stop_revision,
+                            [(other_last_revision, other_revno),
+                             (this_last_revision, this_revno)])
+        self.target.set_last_revision_info(stop_revno, stop_revision)
+
+    @needs_write_lock
     def pull(self, overwrite=False, stop_revision=None,
-             possible_transports=None, _hook_master=None, run_hooks=True,
+             possible_transports=None, run_hooks=True,
              _override_hook_target=None, local=False):
-        """See Branch.pull.
+        """Pull from source into self, updating my master if any.
 
-        :param _hook_master: Private parameter - set the branch to
-            be supplied as the master to pull hooks.
         :param run_hooks: Private parameter - if false, this branch
             is being called because it's the master of the primary branch,
             so it should not run its hooks.
-        :param _override_hook_target: Private parameter - set the branch to be
-            supplied as the target_branch to pull hooks.
-        :param local: Only update the local branch, and not the bound branch.
         """
-        # This type of branch can't be bound.
-        if local:
+        bound_location = self.target.get_bound_location()
+        if local and not bound_location:
             raise errors.LocalRequiresBoundBranch()
-        result = PullResult()
-        result.source_branch = self.source
-        if _override_hook_target is None:
-            result.target_branch = self.target
-        else:
-            result.target_branch = _override_hook_target
-        self.source.lock_read()
+        master_branch = None
+        if not local and bound_location and self.source.user_url != bound_location:
+            # not pulling from master, so we need to update master.
+            master_branch = self.target.get_master_branch(possible_transports)
+            master_branch.lock_write()
         try:
-            # We assume that during 'pull' the target repository is closer than
-            # the source one.
-            self.source.update_references(self.target)
-            graph = self.target.repository.get_graph(self.source.repository)
-            # TODO: Branch formats should have a flag that indicates 
-            # that revno's are expensive, and pull() should honor that flag.
-            # -- JRV20090506
-            result.old_revno, result.old_revid = \
-                self.target.last_revision_info()
-            self.target.update_revisions(self.source, stop_revision,
-                overwrite=overwrite, graph=graph)
-            # TODO: The old revid should be specified when merging tags, 
-            # so a tags implementation that versions tags can only 
-            # pull in the most recent changes. -- JRV20090506
-            result.tag_conflicts = self.source.tags.merge_to(self.target.tags,
-                overwrite)
-            result.new_revno, result.new_revid = self.target.last_revision_info()
-            if _hook_master:
-                result.master_branch = _hook_master
-                result.local_branch = result.target_branch
-            else:
-                result.master_branch = result.target_branch
-                result.local_branch = None
-            if run_hooks:
-                for hook in Branch.hooks['post_pull']:
-                    hook(result)
+            if master_branch:
+                # pull from source into master.
+                master_branch.pull(self.source, overwrite, stop_revision,
+                    run_hooks=False)
+            return self._pull(overwrite,
+                stop_revision, _hook_master=master_branch,
+                run_hooks=run_hooks,
+                _override_hook_target=_override_hook_target)
         finally:
-            self.source.unlock()
-        return result
+            if master_branch:
+                master_branch.unlock()
 
     def push(self, overwrite=False, stop_revision=None,
              _override_hook_source_branch=None):
@@ -3397,48 +3404,63 @@
             _run_hooks()
             return result
 
-    @classmethod
-    def is_compatible(self, source, target):
-        # GenericBranch uses the public API, so always compatible
-        return True
-
-
-class InterToBranch5(GenericInterBranch):
-
-    @staticmethod
-    def _get_branch_formats_to_test():
-        return BranchFormat._default_format, BzrBranchFormat5()
-
-    def pull(self, overwrite=False, stop_revision=None,
-             possible_transports=None, run_hooks=True,
+    def _pull(self, overwrite=False, stop_revision=None,
+             possible_transports=None, _hook_master=None, run_hooks=True,
              _override_hook_target=None, local=False):
-        """Pull from source into self, updating my master if any.
-
+        """See Branch.pull.
+
+        This function is the core worker, used by GenericInterBranch.pull to
+        avoid duplication when pulling source->master and source->local.
+
+        :param _hook_master: Private parameter - set the branch to
+            be supplied as the master to pull hooks.
         :param run_hooks: Private parameter - if false, this branch
             is being called because it's the master of the primary branch,
             so it should not run its hooks.
+        :param _override_hook_target: Private parameter - set the branch to be
+            supplied as the target_branch to pull hooks.
+        :param local: Only update the local branch, and not the bound branch.
         """
-        bound_location = self.target.get_bound_location()
-        if local and not bound_location:
+        # This type of branch can't be bound.
+        if local:
             raise errors.LocalRequiresBoundBranch()
-        master_branch = None
-        if not local and bound_location and self.source.user_url != bound_location:
-            # not pulling from master, so we need to update master.
-            master_branch = self.target.get_master_branch(possible_transports)
-            master_branch.lock_write()
+        result = PullResult()
+        result.source_branch = self.source
+        if _override_hook_target is None:
+            result.target_branch = self.target
+        else:
+            result.target_branch = _override_hook_target
+        self.source.lock_read()
         try:
-            if master_branch:
-                # pull from source into master.
-                master_branch.pull(self.source, overwrite, stop_revision,
-                    run_hooks=False)
-            return super(InterToBranch5, self).pull(overwrite,
-                stop_revision, _hook_master=master_branch,
-                run_hooks=run_hooks,
-                _override_hook_target=_override_hook_target)
+            # We assume that during 'pull' the target repository is closer than
+            # the source one.
+            self.source.update_references(self.target)
+            graph = self.target.repository.get_graph(self.source.repository)
+            # TODO: Branch formats should have a flag that indicates 
+            # that revno's are expensive, and pull() should honor that flag.
+            # -- JRV20090506
+            result.old_revno, result.old_revid = \
+                self.target.last_revision_info()
+            self.target.update_revisions(self.source, stop_revision,
+                overwrite=overwrite, graph=graph)
+            # TODO: The old revid should be specified when merging tags, 
+            # so a tags implementation that versions tags can only 
+            # pull in the most recent changes. -- JRV20090506
+            result.tag_conflicts = self.source.tags.merge_to(self.target.tags,
+                overwrite)
+            result.new_revno, result.new_revid = self.target.last_revision_info()
+            if _hook_master:
+                result.master_branch = _hook_master
+                result.local_branch = result.target_branch
+            else:
+                result.master_branch = result.target_branch
+                result.local_branch = None
+            if run_hooks:
+                for hook in Branch.hooks['post_pull']:
+                    hook(result)
         finally:
-            if master_branch:
-                master_branch.unlock()
+            self.source.unlock()
+        return result
 
 
 InterBranch.register_optimiser(GenericInterBranch)
-InterBranch.register_optimiser(InterToBranch5)

=== modified file 'bzrlib/breakin.py'
--- bzrlib/breakin.py	2010-02-26 03:00:34 +0000
+++ bzrlib/breakin.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -78,7 +78,7 @@
 def hook_debugger_to_signal():
     """Add a signal handler so we drop into the debugger.
 
-    On Linux and Mac, this is hooked into SIGQUIT (C-\\) on Windows, this is
+    On Unix, this is hooked into SIGQUIT (C-\\), and on Windows, this is
     hooked into SIGBREAK (C-Pause).
     """
 

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2010-05-25 17:27:52 +0000
+++ bzrlib/builtins.py	2010-06-07 02:38:04 +0000
@@ -491,7 +491,7 @@
     takes_options = [
         Option('force',
                help='Remove the working tree even if it has '
-                    'uncommitted changes.'),
+                    'uncommitted or shelved changes.'),
         ]
 
     def run(self, location_list, force=False):
@@ -511,6 +511,8 @@
             if not force:
                 if (working.has_changes()):
                     raise errors.UncommittedChanges(working)
+                if working.get_shelf_manager().last_shelf() is not None:
+                    raise errors.ShelvedChanges(working)
 
             if working.user_url != working.branch.user_url:
                 raise errors.BzrCommandError("You cannot remove the working tree"
@@ -3309,7 +3311,8 @@
 
             bzr whoami "Frank Chu <fchu@example.com>"
     """
-    takes_options = [ Option('email',
+    takes_options = [ 'directory',
+                      Option('email',
                              help='Display email address only.'),
                       Option('branch',
                              help='Set identity for the current branch instead of '
@@ -3319,13 +3322,16 @@
     encoding_type = 'replace'
 
     @display_command
-    def run(self, email=False, branch=False, name=None):
+    def run(self, email=False, branch=False, name=None, directory=None):
         if name is None:
-            # use branch if we're inside one; otherwise global config
-            try:
-                c = Branch.open_containing('.')[0].get_config()
-            except errors.NotBranchError:
-                c = config.GlobalConfig()
+            if directory is None:
+                # use branch if we're inside one; otherwise global config
+                try:
+                    c = Branch.open_containing(u'.')[0].get_config()
+                except errors.NotBranchError:
+                    c = config.GlobalConfig()
+            else:
+                c = Branch.open(directory).get_config()
             if email:
                 self.outf.write(c.user_email() + '\n')
             else:
@@ -3341,7 +3347,10 @@
 
         # use global config unless --branch given
         if branch:
-            c = Branch.open_containing('.')[0].get_config()
+            if directory is None:
+                c = Branch.open_containing(u'.')[0].get_config()
+            else:
+                c = Branch.open(directory).get_config()
         else:
             c = config.GlobalConfig()
         c.set_user_option('email', name)
@@ -4289,6 +4298,7 @@
     _see_also = ['merge', 'pull']
     takes_args = ['other_branch?']
     takes_options = [
+        'directory',
         Option('reverse', 'Reverse the order of revisions.'),
         Option('mine-only',
                'Display changes in the local branch only.'),
@@ -4316,7 +4326,8 @@
             theirs_only=False,
             log_format=None, long=False, short=False, line=False,
             show_ids=False, verbose=False, this=False, other=False,
-            include_merges=False, revision=None, my_revision=None):
+            include_merges=False, revision=None, my_revision=None,
+            directory=u'.'):
         from bzrlib.missing import find_unmerged, iter_log_revisions
         def message(s):
             if not is_quiet():
@@ -4335,7 +4346,7 @@
         elif theirs_only:
             restrict = 'remote'
 
-        local_branch = Branch.open_containing(u".")[0]
+        local_branch = Branch.open_containing(directory)[0]
         self.add_cleanup(local_branch.lock_read().unlock)
 
         parent = local_branch.get_parent()
@@ -5012,6 +5023,7 @@
     _see_also = ['send']
 
     takes_options = [
+        'directory',
         RegistryOption.from_kwargs('patch-type',
             'The type of patch to include in the directive.',
             title='Patch type',
@@ -5030,14 +5042,15 @@
     encoding_type = 'exact'
 
     def run(self, submit_branch=None, public_branch=None, patch_type='bundle',
-            sign=False, revision=None, mail_to=None, message=None):
+            sign=False, revision=None, mail_to=None, message=None,
+            directory=u'.'):
         from bzrlib.revision import ensure_null, NULL_REVISION
         include_patch, include_bundle = {
             'plain': (False, False),
             'diff': (True, False),
             'bundle': (True, True),
             }[patch_type]
-        branch = Branch.open('.')
+        branch = Branch.open(directory)
         stored_submit_branch = branch.get_submit_branch()
         if submit_branch is None:
             submit_branch = stored_submit_branch
@@ -5128,7 +5141,7 @@
     given, in which case it is sent to a file.
 
     Mail is sent using your preferred mail program.  This should be transparent
-    on Windows (it uses MAPI).  On Linux, it requires the xdg-email utility.
+    on Windows (it uses MAPI).  On Unix, it requires the xdg-email utility.
     If the preferred client can't be found (or used), your editor will be used.
 
     To use a specific mail program, set the mail_client configuration option.
@@ -5531,7 +5544,8 @@
     """
 
     takes_args = ['to_location?']
-    takes_options = [Option('force',
+    takes_options = ['directory',
+                     Option('force',
                         help='Switch even if local commits will be lost.'),
                      'revision',
                      Option('create-branch', short_name='b',
@@ -5540,16 +5554,16 @@
                     ]
 
     def run(self, to_location=None, force=False, create_branch=False,
-            revision=None):
+            revision=None, directory=u'.'):
         from bzrlib import switch
-        tree_location = '.'
+        tree_location = directory
         revision = _get_one_revision('switch', revision)
         control_dir = bzrdir.BzrDir.open_containing(tree_location)[0]
         if to_location is None:
             if revision is None:
                 raise errors.BzrCommandError('You must supply either a'
                                              ' revision or a location')
-            to_location = '.'
+            to_location = tree_location
         try:
             branch = control_dir.open_branch()
             had_explicit_nick = branch.get_config().has_explicit_nickname()
@@ -5830,6 +5844,7 @@
     takes_args = ['file*']
 
     takes_options = [
+        'directory',
         'revision',
         Option('all', help='Shelve all changes.'),
         'message',
@@ -5844,7 +5859,7 @@
     _see_also = ['unshelve']
 
     def run(self, revision=None, all=False, file_list=None, message=None,
-            writer=None, list=False, destroy=False):
+            writer=None, list=False, destroy=False, directory=u'.'):
         if list:
             return self.run_for_list()
         from bzrlib.shelf_ui import Shelver
@@ -5852,7 +5867,7 @@
             writer = bzrlib.option.diff_writer_registry.get()
         try:
             shelver = Shelver.from_args(writer(sys.stdout), revision, all,
-                file_list, message, destroy=destroy)
+                file_list, message, destroy=destroy, directory=directory)
             try:
                 shelver.run()
             finally:
@@ -5886,6 +5901,7 @@
 
     takes_args = ['shelf_id?']
     takes_options = [
+        'directory',
         RegistryOption.from_kwargs(
             'action', help="The action to perform.",
             enum_switch=False, value_switches=True,
@@ -5899,9 +5915,9 @@
     ]
     _see_also = ['shelve']
 
-    def run(self, shelf_id=None, action='apply'):
+    def run(self, shelf_id=None, action='apply', directory=u'.'):
         from bzrlib.shelf_ui import Unshelver
-        unshelver = Unshelver.from_args(shelf_id, action)
+        unshelver = Unshelver.from_args(shelf_id, action, directory=directory)
         try:
             unshelver.run()
         finally:

=== modified file 'bzrlib/bundle/bundle_data.py'
--- bzrlib/bundle/bundle_data.py	2009-09-20 22:12:36 +0000
+++ bzrlib/bundle/bundle_data.py	2010-06-16 12:47:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -331,7 +331,7 @@
                 try:
                     name, value = info_item.split(':', 1)
                 except ValueError:
-                    raise 'Value %r has no colon' % info_item
+                    raise ValueError('Value %r has no colon' % info_item)
                 if name == 'last-changed':
                     last_changed = value
                 elif name == 'executable':

=== modified file 'bzrlib/cleanup.py'
--- bzrlib/cleanup.py	2010-03-03 22:59:21 +0000
+++ bzrlib/cleanup.py	2010-04-27 07:29:11 +0000
@@ -78,7 +78,28 @@
         _run_cleanup(func, *args, **kwargs)
 
 
-class OperationWithCleanups(object):
+class ObjectWithCleanups(object):
+    """A mixin for objects that hold a cleanup list.
+
+    Subclass or client code can call add_cleanup and then later `cleanup_now`.
+    """
+    def __init__(self):
+        self.cleanups = deque()
+
+    def add_cleanup(self, cleanup_func, *args, **kwargs):
+        """Add a cleanup to run.
+
+        Cleanups may be added at any time.  
+        Cleanups will be executed in LIFO order.
+        """
+        self.cleanups.appendleft((cleanup_func, args, kwargs))
+
+    def cleanup_now(self):
+        _run_cleanups(self.cleanups)
+        self.cleanups.clear()
+
+
+class OperationWithCleanups(ObjectWithCleanups):
     """A way to run some code with a dynamic cleanup list.
 
     This provides a way to add cleanups while the function-with-cleanups is
@@ -102,16 +123,8 @@
     """
 
     def __init__(self, func):
+        super(OperationWithCleanups, self).__init__()
         self.func = func
-        self.cleanups = deque()
-
-    def add_cleanup(self, cleanup_func, *args, **kwargs):
-        """Add a cleanup to run.
-
-        Cleanups may be added at any time before or during the execution of
-        self.func.  Cleanups will be executed in LIFO order.
-        """
-        self.cleanups.appendleft((cleanup_func, args, kwargs))
 
     def run(self, *args, **kwargs):
         return _do_with_cleanups(
@@ -121,10 +134,6 @@
         return _do_with_cleanups(
             self.cleanups, self.func, *args, **kwargs)
 
-    def cleanup_now(self):
-        _run_cleanups(self.cleanups)
-        self.cleanups.clear()
-
 
 def _do_with_cleanups(cleanup_funcs, func, *args, **kwargs):
     """Run `func`, then call all the cleanup_funcs.

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2010-05-25 17:27:52 +0000
+++ bzrlib/config.py	2010-06-02 04:50:35 +0000
@@ -843,7 +843,6 @@
                                   ' or HOME set')
         return osutils.pathjoin(base, 'bazaar', '2.0')
     else:
-        # cygwin, linux, and darwin all have a $HOME directory
         if base is None:
             base = os.path.expanduser("~")
         return osutils.pathjoin(base, ".bazaar")

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2010-04-30 11:35:43 +0000
+++ bzrlib/conflicts.py	2010-05-28 14:15:28 +0000
@@ -59,13 +59,14 @@
     Use bzr resolve when you have fixed a problem.
     """
     takes_options = [
+            'directory',
             option.Option('text',
                           help='List paths of files with text conflicts.'),
         ]
     _see_also = ['resolve', 'conflict-types']
 
-    def run(self, text=False):
-        wt = workingtree.WorkingTree.open_containing(u'.')[0]
+    def run(self, text=False, directory=u'.'):
+        wt = workingtree.WorkingTree.open_containing(directory)[0]
         for conflict in wt.conflicts():
             if text:
                 if conflict.typestring != 'text conflict':
@@ -112,16 +113,17 @@
     aliases = ['resolved']
     takes_args = ['file*']
     takes_options = [
+            'directory',
             option.Option('all', help='Resolve all conflicts in this tree.'),
             ResolveActionOption(),
             ]
     _see_also = ['conflicts']
-    def run(self, file_list=None, all=False, action=None):
+    def run(self, file_list=None, all=False, action=None, directory=u'.'):
         if all:
             if file_list:
                 raise errors.BzrCommandError("If --all is specified,"
                                              " no FILE may be provided")
-            tree = workingtree.WorkingTree.open_containing('.')[0]
+            tree = workingtree.WorkingTree.open_containing(directory)[0]
             if action is None:
                 action = 'done'
         else:

=== modified file 'bzrlib/doc_generate/autodoc_man.py'
--- bzrlib/doc_generate/autodoc_man.py	2010-02-23 07:43:11 +0000
+++ bzrlib/doc_generate/autodoc_man.py	2010-06-02 04:50:52 +0000
@@ -186,7 +186,7 @@
 .B "help"
 .I "command"
 .SH "DESCRIPTION"
-Bazaar (or %(bzrcmd)s) is a project of Canonical to develop an open source
+Bazaar (or %(bzrcmd)s) is a project of Canonical to develop an free
 distributed version control system that is powerful, friendly, and scalable.
 Version control means a system that keeps track of previous revisions
 of software source code or similar information and helps people work on it in teams.

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2010-05-13 17:32:55 +0000
+++ bzrlib/errors.py	2010-05-28 06:04:57 +0000
@@ -2850,6 +2850,12 @@
         BzrError.__init__(self, tree=tree, display_url=display_url, more=more)
 
 
+class ShelvedChanges(UncommittedChanges):
+
+    _fmt = ('Working tree "%(display_url)s" has shelved changes'
+            ' (See bzr shelve --list).%(more)s')
+
+
 class MissingTemplateVariable(BzrError):
 
     _fmt = 'Variable {%(name)s} is not available.'

=== modified file 'bzrlib/filters/eol.py'
--- bzrlib/filters/eol.py	2009-05-07 05:08:46 +0000
+++ bzrlib/filters/eol.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,8 +25,8 @@
 from bzrlib.errors import BzrError
 
 
-# Real Linux/Unix/OSX newline - \n without \r before it
-_LINUX_NL_RE = re.compile(r'(?<!\r)\n')
+# Real Unix newline - \n without \r before it
+_UNIX_NL_RE = re.compile(r'(?<!\r)\n')
 
 
 def _to_lf_converter(chunks, context=None):
@@ -44,7 +44,7 @@
     if '\x00' in content:
         return [content]
     else:
-        return [_LINUX_NL_RE.sub('\r\n', content)]
+        return [_UNIX_NL_RE.sub('\r\n', content)]
 
 
 # Register the eol content filter.

=== modified file 'bzrlib/help_topics/__init__.py'
--- bzrlib/help_topics/__init__.py	2010-05-20 11:04:58 +0000
+++ bzrlib/help_topics/__init__.py	2010-06-02 04:50:35 +0000
@@ -618,7 +618,7 @@
 _files = \
 r"""Files
 
-:On Linux:   ~/.bazaar/bazaar.conf
+:On Unix:   ~/.bazaar/bazaar.conf
 :On Windows: C:\\Documents and Settings\\username\\Application Data\\bazaar\\2.0\\bazaar.conf
 
 Contains the user's default configuration. The section ``[DEFAULT]`` is

=== modified file 'bzrlib/help_topics/en/configuration.txt'
--- bzrlib/help_topics/en/configuration.txt	2010-05-04 14:52:33 +0000
+++ bzrlib/help_topics/en/configuration.txt	2010-06-02 05:03:31 +0000
@@ -90,7 +90,7 @@
 
 As for the ``PATH`` variables, if multiple directories are
 specified in ``BZR_PLUGIN_PATH`` they should be separated by the
-platform specific appropriate character (':' on Unix/Linux/etc,
+platform specific appropriate character (':' on Unix,
 ';' on windows)
 
 By default if ``BZR_PLUGIN_PATH`` is set, it replaces searching
@@ -185,7 +185,7 @@
 Location
 ~~~~~~~~
 
-Configuration files are located in ``$HOME/.bazaar`` on Linux/Unix and
+Configuration files are located in ``$HOME/.bazaar`` on Unix and
 ``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0`` on
 Windows. (You can check the location for your system by using
 ``bzr version``.)

=== modified file 'bzrlib/help_topics/en/content-filters.txt'
--- bzrlib/help_topics/en/content-filters.txt	2010-01-03 03:33:10 +0000
+++ bzrlib/help_topics/en/content-filters.txt	2010-06-02 05:03:31 +0000
@@ -8,7 +8,7 @@
 format from the copy in your working tree.  This lets you, or your
 co-developers, use Windows development tools that expect CRLF files
 on projects that use other line-ending conventions. Among other things,
-content filters also let Linux developers more easily work on projects
+content filters also let Unix developers more easily work on projects
 using Windows line-ending conventions, keyword expansion/compression,
 and trailing spaces on lines in text files to be implicitly stripped
 when committed.

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2010-05-14 13:25:05 +0000
+++ bzrlib/knit.py	2010-06-04 03:09:35 +0000
@@ -68,6 +68,7 @@
     index as _mod_index,
     lru_cache,
     pack,
+    patiencediff,
     progress,
     static_tuple,
     trace,
@@ -79,7 +80,6 @@
 from bzrlib import (
     errors,
     osutils,
-    patiencediff,
     )
 from bzrlib.errors import (
     FileExists,
@@ -3417,10 +3417,6 @@
             raise exc_class, exc_value, exc_traceback
 
 
-# Deprecated, use PatienceSequenceMatcher instead
-KnitSequenceMatcher = patiencediff.PatienceSequenceMatcher
-
-
 def annotate_knit(knit, revision_id):
     """Annotate a knit with no cached annotations.
 

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2010-05-11 08:44:59 +0000
+++ bzrlib/log.py	2010-06-08 09:50:27 +0000
@@ -522,7 +522,7 @@
     elif not generate_merge_revisions:
         # If we only want to see linear revisions, we can iterate ...
         iter_revs = _generate_flat_revisions(branch, start_rev_id, end_rev_id,
-                                             direction)
+                                             direction, exclude_common_ancestry)
         if direction == 'forward':
             iter_revs = reversed(iter_revs)
     else:
@@ -544,8 +544,11 @@
         return [(rev_id, revno_str, 0)]
 
 
-def _generate_flat_revisions(branch, start_rev_id, end_rev_id, direction):
-    result = _linear_view_revisions(branch, start_rev_id, end_rev_id)
+def _generate_flat_revisions(branch, start_rev_id, end_rev_id, direction,
+                             exclude_common_ancestry=False):
+    result = _linear_view_revisions(
+        branch, start_rev_id, end_rev_id,
+        exclude_common_ancestry=exclude_common_ancestry)
     # If a start limit was given and it's not obviously an
     # ancestor of the end limit, check it before outputting anything
     if direction == 'forward' or (start_rev_id
@@ -572,7 +575,7 @@
     if delayed_graph_generation:
         try:
             for rev_id, revno, depth in  _linear_view_revisions(
-                branch, start_rev_id, end_rev_id):
+                branch, start_rev_id, end_rev_id, exclude_common_ancestry):
                 if _has_merges(branch, rev_id):
                     # The end_rev_id can be nested down somewhere. We need an
                     # explicit ancestry check. There is an ambiguity here as we
@@ -643,14 +646,17 @@
     return True
 
 
-def _linear_view_revisions(branch, start_rev_id, end_rev_id):
+def _linear_view_revisions(branch, start_rev_id, end_rev_id,
+                           exclude_common_ancestry=False):
     """Calculate a sequence of revisions to view, newest to oldest.
 
     :param start_rev_id: the lower revision-id
     :param end_rev_id: the upper revision-id
+    :param exclude_common_ancestry: Whether the start_rev_id should be part of
+        the iterated revisions.
     :return: An iterator of (revision_id, dotted_revno, merge_depth) tuples.
     :raises _StartNotLinearAncestor: if a start_rev_id is specified but
-      is not found walking the left-hand history
+        is not found walking the left-hand history
     """
     br_revno, br_rev_id = branch.last_revision_info()
     repo = branch.repository
@@ -667,7 +673,8 @@
             revno = branch.revision_id_to_dotted_revno(revision_id)
             revno_str = '.'.join(str(n) for n in revno)
             if not found_start and revision_id == start_rev_id:
-                yield revision_id, revno_str, 0
+                if not exclude_common_ancestry:
+                    yield revision_id, revno_str, 0
                 found_start = True
                 break
             else:

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2010-05-25 17:27:52 +0000
+++ bzrlib/merge.py	2010-06-04 03:09:35 +0000
@@ -16,14 +16,14 @@
 
 import warnings
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 from bzrlib import (
     branch as _mod_branch,
     conflicts as _mod_conflicts,
     debug,
-    decorators,
     errors,
     graph as _mod_graph,
-    hooks,
     merge3,
     osutils,
     patiencediff,
@@ -34,9 +34,14 @@
     tree as _mod_tree,
     tsort,
     ui,
-    versionedfile
+    versionedfile,
     )
 from bzrlib.cleanup import OperationWithCleanups
+""")
+from bzrlib import (
+    decorators,
+    hooks,
+    )
 from bzrlib.symbol_versioning import (
     deprecated_in,
     deprecated_method,

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2010-06-02 11:51:58 +0000
+++ bzrlib/osutils.py	2010-06-17 09:23:19 +0000
@@ -1308,7 +1308,7 @@
 def normalizes_filenames():
     """Return True if this platform normalizes unicode filenames.
 
-    Mac OSX does, Windows/Linux do not.
+    Only Mac OSX.
     """
     return _platform_normalizes_filenames
 
@@ -1319,7 +1319,7 @@
     On platforms where the system normalizes filenames (Mac OSX),
     you can access a file by any path which will normalize correctly.
     On platforms where the system does not normalize filenames
-    (Windows, Linux), you have to access a file by its exact path.
+    (everything else), you have to access a file by its exact path.
 
     Internally, bzr only supports NFC normalization, since that is
     the standard for XML documents.

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2010-04-06 07:22:31 +0000
+++ bzrlib/plugin.py	2010-06-11 08:02:42 +0000
@@ -81,6 +81,33 @@
     return path.rstrip("\\/")
 
 
+def _get_specific_plugin_paths(paths):
+    """Returns the plugin paths from a string describing the associations.
+
+    :param paths: A string describing the paths associated with the plugins.
+
+    :returns: A list of (plugin name, path) tuples.
+
+    For example, if paths is my_plugin@/test/my-test:her_plugin@/production/her,
+    [('my_plugin', '/test/my-test'), ('her_plugin', '/production/her')] 
+    will be returned.
+
+    Note that ':' in the example above depends on the os.
+    """
+    if not paths:
+        return []
+    specs = []
+    for spec in paths.split(os.pathsep):
+        try:
+            name, path = spec.split('@')
+        except ValueError:
+            raise errors.BzrCommandError(
+                '"%s" is not a valid <plugin_name>@<plugin_path> description '
+                % spec)
+        specs.append((name, path))
+    return specs
+
+
 def set_plugins_path(path=None):
     """Set the path for plugins to be loaded from.
 
@@ -98,10 +125,8 @@
         for name in disabled_plugins.split(os.pathsep):
             PluginImporter.blacklist.add('bzrlib.plugins.' + name)
     # Set up a the specific paths for plugins
-    specific_plugins = os.environ.get('BZR_PLUGINS_AT', None)
-    if specific_plugins is not None:
-        for spec in specific_plugins.split(os.pathsep):
-            plugin_name, plugin_path = spec.split('@')
+    for plugin_name, plugin_path in _get_specific_plugin_paths(os.environ.get(
+            'BZR_PLUGINS_AT', None)):
             PluginImporter.specific_paths[
                 'bzrlib.plugins.%s' % plugin_name] = plugin_path
     return path
@@ -562,7 +587,6 @@
         # We are called only for specific paths
         plugin_path = self.specific_paths[fullname]
         loading_path = None
-        package = False
         if os.path.isdir(plugin_path):
             for suffix, mode, kind in imp.get_suffixes():
                 if kind not in (imp.PY_SOURCE, imp.PY_COMPILED):
@@ -570,8 +594,12 @@
                     continue
                 init_path = osutils.pathjoin(plugin_path, '__init__' + suffix)
                 if os.path.isfile(init_path):
-                    loading_path = init_path
-                    package = True
+                    # We've got a module here and load_module needs specific
+                    # parameters.
+                    loading_path = plugin_path
+                    suffix = ''
+                    mode = ''
+                    kind = imp.PKG_DIRECTORY
                     break
         else:
             for suffix, mode, kind in imp.get_suffixes():
@@ -581,17 +609,18 @@
         if loading_path is None:
             raise ImportError('%s cannot be loaded from %s'
                               % (fullname, plugin_path))
-        f = open(loading_path, mode)
+        if kind is imp.PKG_DIRECTORY:
+            f = None
+        else:
+            f = open(loading_path, mode)
         try:
             mod = imp.load_module(fullname, f, loading_path,
                                   (suffix, mode, kind))
-            if package:
-                # The plugin can contain modules, so be ready
-                mod.__path__ = [plugin_path]
             mod.__package__ = fullname
             return mod
         finally:
-            f.close()
+            if f is not None:
+                f.close()
 
 
 # Install a dedicated importer for plugins requiring special handling

=== modified file 'bzrlib/plugins/bash_completion/bashcomp.py'
--- bzrlib/plugins/bash_completion/bashcomp.py	2010-05-21 06:17:55 +0000
+++ bzrlib/plugins/bash_completion/bashcomp.py	2010-05-29 08:47:10 +0000
@@ -66,12 +66,13 @@
 {
 	local cur cmds cmdIdx cmd cmdOpts fixedWords i globalOpts
 	local curOpt optEnums
+	local IFS=$' \\n'
 
 	COMPREPLY=()
 	cur=${COMP_WORDS[COMP_CWORD]}
 
 	cmds='%(cmds)s'
-	globalOpts='%(global_options)s'
+	globalOpts=( %(global_options)s )
 
 	# do ordinary expansion if we are anywhere after a -- argument
 	for ((i = 1; i < COMP_CWORD; ++i)); do
@@ -89,7 +90,7 @@
 
 	# complete command name if we are not already past the command
 	if [[ $COMP_CWORD -le cmdIdx ]]; then
-		COMPREPLY=( $( compgen -W "$cmds $globalOpts" -- $cur ) )
+		COMPREPLY=( $( compgen -W "$cmds ${globalOpts[*]}" -- $cur ) )
 		return 0
 	fi
 
@@ -107,32 +108,45 @@
 		fi
 	fi
 %(debug)s
-	cmdOpts=
-	optEnums=
-	fixedWords=
+	cmdOpts=( )
+	optEnums=( )
+	fixedWords=( )
 	case $cmd in
 %(cases)s\
 	*)
-		cmdOpts='--help -h'
+		cmdOpts=(--help -h)
 		;;
 	esac
 
-	if [[ -z $fixedWords ]] && [[ -z $optEnums ]] && [[ $cur != -* ]]; then
+	IFS=$'\\n'
+	if [[ ${#fixedWords[@]} -eq 0 ]] && [[ ${#optEnums[@]} -eq 0 ]] && [[ $cur != -* ]]; then
 		case $curOpt in
-			tag:*)
-				fixedWords="$(bzr tags 2>/dev/null | sed 's/  *[^ ]*$//')"
-				;;
-		esac
-	elif [[ $cur == = ]] && [[ -n $optEnums ]]; then
+			tag:|*..tag:)
+				fixedWords=( $(bzr tags 2>/dev/null | sed 's/  *[^ ]*$//; s/ /\\\\\\\\ /g;') )
+				;;
+		esac
+		case $cur in
+			[\\"\\']tag:*)
+				fixedWords=( $(bzr tags 2>/dev/null | sed 's/  *[^ ]*$//; s/^/tag:/') )
+				;;
+			[\\"\\']*..tag:*)
+				fixedWords=( $(bzr tags 2>/dev/null | sed 's/  *[^ ]*$//') )
+				fixedWords=( $(for i in "${fixedWords[@]}"; do echo "${cur%%..tag:*}..tag:${i}"; done) )
+				;;
+		esac
+	elif [[ $cur == = ]] && [[ ${#optEnums[@]} -gt 0 ]]; then
 		# complete directly after "--option=", list all enum values
-		COMPREPLY=( $optEnums )
+		COMPREPLY=( "${optEnums[@]}" )
 		return 0
 	else
-		fixedWords="$cmdOpts $globalOpts $optEnums $fixedWords"
+		fixedWords=( "${cmdOpts[@]}"
+		             "${globalOpts[@]}"
+		             "${optEnums[@]}"
+		             "${fixedWords[@]}" )
 	fi
 
-	if [[ -n $fixedWords ]]; then
-		COMPREPLY=( $( compgen -W "$fixedWords" -- $cur ) )
+	if [[ ${#fixedWords[@]} -gt 0 ]]; then
+		COMPREPLY=( $( compgen -W "${fixedWords[*]}" -- $cur ) )
 	fi
 
 	return 0
@@ -144,6 +158,7 @@
             "global_options": self.global_options(),
             "debug": self.debug_output(),
         })
+        # Help Emacs terminate strings: "
 
     def command_names(self):
         return " ".join(self.data.all_command_aliases())
@@ -196,15 +211,15 @@
             if option.registry_keys:
                 for key in option.registry_keys:
                     options.append("%s=%s" % (option, key))
-                enums.append("%s) optEnums='%s' ;;" %
+                enums.append("%s) optEnums=( %s ) ;;" %
                              (option, ' '.join(option.registry_keys)))
             else:
                 options.append(str(option))
-        case += "\t\tcmdOpts='%s'\n" % " ".join(options)
+        case += "\t\tcmdOpts=( %s )\n" % " ".join(options)
         if command.fixed_words:
             fixed_words = command.fixed_words
             if isinstance(fixed_words, list):
-                fixed_words = "'%s'" + ' '.join(fixed_words)
+                fixed_words = "( %s )" + ' '.join(fixed_words)
             case += "\t\tfixedWords=%s\n" % fixed_words
         if enums:
             case += "\t\tcase $curOpt in\n\t\t\t"
@@ -241,7 +256,10 @@
 
     def __init__(self, name, version=None):
         if version is None:
-            version = bzrlib.plugin.plugins()[name].__version__
+            try:
+                version = bzrlib.plugin.plugins()[name].__version__
+            except:
+                version = 'unknown'
         self.name = name
         self.version = version
 
@@ -335,7 +353,7 @@
             cmd_data.options.extend(self.option(opt))
 
         if 'help' == name or 'help' in cmd.aliases:
-            cmd_data.fixed_words = ('"$cmds %s"' %
+            cmd_data.fixed_words = ('($cmds %s)' %
                 " ".join(sorted(help_topics.topic_registry.keys())))
 
         return cmd_data

=== modified file 'bzrlib/plugins/bash_completion/tests/test_bashcomp.py'
--- bzrlib/plugins/bash_completion/tests/test_bashcomp.py	2010-05-25 11:20:16 +0000
+++ bzrlib/plugins/bash_completion/tests/test_bashcomp.py	2010-05-29 10:30:53 +0000
@@ -16,6 +16,7 @@
 
 import bzrlib
 from bzrlib import commands, tests
+from bzrlib.tests import features
 from bzrlib.plugins.bash_completion.bashcomp import *
 
 import os
@@ -25,7 +26,7 @@
 class BashCompletionMixin(object):
     """Component for testing execution of a bash completion script."""
 
-    _test_needs_features = [tests.features.bash_feature]
+    _test_needs_features = [features.bash_feature]
 
     def complete(self, words, cword=-1):
         """Perform a bash completion.
@@ -35,7 +36,7 @@
         """
         if self.script is None:
             self.script = self.get_script()
-        proc = subprocess.Popen([tests.features.bash_feature.path,
+        proc = subprocess.Popen([features.bash_feature.path,
                                  '--noprofile'],
                                 stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE,
@@ -163,7 +164,7 @@
         return s.replace("$(bzr ", "$('%s' " % self.get_bzr_path())
 
     def test_revspec_tag_all(self):
-        self.requireFeature(tests.features.sed_feature)
+        self.requireFeature(features.sed_feature)
         wt = self.make_branch_and_tree('.', format='dirstate-tags')
         wt.branch.tags.set_tag('tag1', 'null:')
         wt.branch.tags.set_tag('tag2', 'null:')
@@ -172,7 +173,7 @@
         self.assertCompletionEquals('tag1', 'tag2', '3tag')
 
     def test_revspec_tag_prefix(self):
-        self.requireFeature(tests.features.sed_feature)
+        self.requireFeature(features.sed_feature)
         wt = self.make_branch_and_tree('.', format='dirstate-tags')
         wt.branch.tags.set_tag('tag1', 'null:')
         wt.branch.tags.set_tag('tag2', 'null:')
@@ -180,6 +181,29 @@
         self.complete(['bzr', 'log', '-r', 'tag', ':', 't'])
         self.assertCompletionEquals('tag1', 'tag2')
 
+    def test_revspec_tag_spaces(self):
+        self.requireFeature(features.sed_feature)
+        wt = self.make_branch_and_tree('.', format='dirstate-tags')
+        wt.branch.tags.set_tag('tag with spaces', 'null:')
+        self.complete(['bzr', 'log', '-r', 'tag', ':', 't'])
+        self.assertCompletionEquals(r'tag\ with\ spaces')
+        self.complete(['bzr', 'log', '-r', '"tag:t'])
+        self.assertCompletionEquals('tag:tag with spaces')
+        self.complete(['bzr', 'log', '-r', "'tag:t"])
+        self.assertCompletionEquals('tag:tag with spaces')
+
+    def test_revspec_tag_endrange(self):
+        self.requireFeature(features.sed_feature)
+        wt = self.make_branch_and_tree('.', format='dirstate-tags')
+        wt.branch.tags.set_tag('tag1', 'null:')
+        wt.branch.tags.set_tag('tag2', 'null:')
+        self.complete(['bzr', 'log', '-r', '3..tag', ':', 't'])
+        self.assertCompletionEquals('tag1', 'tag2')
+        self.complete(['bzr', 'log', '-r', '"3..tag:t'])
+        self.assertCompletionEquals('3..tag:tag1', '3..tag:tag2')
+        self.complete(['bzr', 'log', '-r', "'3..tag:t"])
+        self.assertCompletionEquals('3..tag:tag1', '3..tag:tag2')
+
 
 class TestBashCodeGen(tests.TestCase):
 
@@ -226,10 +250,10 @@
         cg = BashCodeGen(data)
         self.assertEqualDiff('''\
 \tbar|baz)
-\t\tcmdOpts='--opt'
+\t\tcmdOpts=( --opt )
 \t\t;;
 \tfoo)
-\t\tcmdOpts=''
+\t\tcmdOpts=(  )
 \t\t;;
 ''', cg.command_cases())
 
@@ -248,9 +272,9 @@
 \tcmd)
 \t\t# plugin "plugger 1.0"
 \t\t# Some error message
-\t\tcmdOpts='--bar=that --bar=this --foo'
+\t\tcmdOpts=( --bar=that --bar=this --foo )
 \t\tcase $curOpt in
-\t\t\t--bar) optEnums='that this' ;;
+\t\t\t--bar) optEnums=( that this ) ;;
 \t\tesac
 \t\t;;
 ''', cg.command_case(cmd))
@@ -274,6 +298,12 @@
         self.assertSubset(['init', 'init-repo', 'init-repository'],
                            dc.data.all_command_aliases())
 
+    def test_commands_from_plugins(self):
+        dc = DataCollector()
+        dc.commands()
+        self.assertSubset(['bash-completion'],
+                           dc.data.all_command_aliases())
+
     def test_commit_dashm(self):
         dc = DataCollector()
         cmd = dc.command('commit')

=== modified file 'bzrlib/recordcounter.py'
--- bzrlib/recordcounter.py	2010-05-19 13:30:09 +0000
+++ bzrlib/recordcounter.py	2010-06-12 02:58:42 +0000
@@ -31,10 +31,10 @@
 
         # Users of RecordCounter instance update progress bar every
         # _STEP_ records. We choose are reasonably high number to keep
-        # display updates from being two frequent. This is an odd number
+        # display updates from being too frequent. This is an odd number
         # to ensure that the last digit of the records fetched in
         # fetches vs estimate ratio changes periodically.
-        self.STEP = 71
+        self.STEP = 7
 
     def is_initialized(self):
         return self.initialized

=== modified file 'bzrlib/smart/medium.py'
--- bzrlib/smart/medium.py	2010-05-03 04:08:50 +0000
+++ bzrlib/smart/medium.py	2010-06-16 07:45:53 +0000
@@ -715,10 +715,6 @@
     """A client medium using simple pipes.
 
     This client does not manage the pipes: it assumes they will always be open.
-
-    Note that if readable_pipe.read might raise IOError or OSError with errno
-    of EINTR, it must be safe to retry the read.  Plain CPython fileobjects
-    (such as used for sys.stdin) are safe.
     """
 
     def __init__(self, readable_pipe, writeable_pipe, base):
@@ -737,26 +733,40 @@
 
     def _read_bytes(self, count):
         """See SmartClientStreamMedium._read_bytes."""
-        bytes = osutils.until_no_eintr(self._readable_pipe.read, count)
+        bytes_to_read = min(count, _MAX_READ_SIZE)
+        bytes = self._readable_pipe.read(bytes_to_read)
         self._report_activity(len(bytes), 'read')
         return bytes
 
 
+class SSHParams(object):
+    """A set of parameters for starting a remote bzr via SSH."""
+
+    def __init__(self, host, port=None, username=None, password=None,
+            bzr_remote_path='bzr'):
+        self.host = host
+        self.port = port
+        self.username = username
+        self.password = password
+        self.bzr_remote_path = bzr_remote_path
+
+
 class SmartSSHClientMedium(SmartClientStreamMedium):
-    """A client medium using SSH."""
+    """A client medium using SSH.
+    
+    It delegates IO to a SmartClientSocketMedium or
+    SmartClientAlreadyConnectedSocketMedium (depending on platform).
+    """
 
-    def __init__(self, host, port=None, username=None, password=None,
-            base=None, vendor=None, bzr_remote_path=None):
+    def __init__(self, base, ssh_params, vendor=None):
         """Creates a client that will connect on the first use.
 
+        :param ssh_params: A SSHParams instance.
         :param vendor: An optional override for the ssh vendor to use. See
             bzrlib.transport.ssh for details on ssh vendors.
         """
-        self._connected = False
-        self._host = host
-        self._password = password
-        self._port = port
-        self._username = username
+        self._real_medium = None
+        self._ssh_params = ssh_params
         # for the benefit of progress making a short description of this
         # transport
         self._scheme = 'bzr+ssh'
@@ -764,67 +774,70 @@
         # _DebugCounter so we have to store all the values used in our repr
         # method before calling the super init.
         SmartClientStreamMedium.__init__(self, base)
-        self._read_from = None
+        self._vendor = vendor
         self._ssh_connection = None
-        self._vendor = vendor
-        self._write_to = None
-        self._bzr_remote_path = bzr_remote_path
 
     def __repr__(self):
-        if self._port is None:
+        if self._ssh_params.port is None:
             maybe_port = ''
         else:
-            maybe_port = ':%s' % self._port
+            maybe_port = ':%s' % self._ssh_params.port
         return "%s(%s://%s@%s%s/)" % (
             self.__class__.__name__,
             self._scheme,
-            self._username,
-            self._host,
+            self._ssh_params.username,
+            self._ssh_params.host,
             maybe_port)
 
     def _accept_bytes(self, bytes):
         """See SmartClientStreamMedium.accept_bytes."""
         self._ensure_connection()
-        self._write_to.write(bytes)
-        self._report_activity(len(bytes), 'write')
+        self._real_medium.accept_bytes(bytes)
 
     def disconnect(self):
         """See SmartClientMedium.disconnect()."""
-        if not self._connected:
-            return
-        self._read_from.close()
-        self._write_to.close()
-        self._ssh_connection.close()
-        self._connected = False
+        if self._real_medium is not None:
+            self._real_medium.disconnect()
+            self._real_medium = None
+        if self._ssh_connection is not None:
+            self._ssh_connection.close()
+            self._ssh_connection = None
 
     def _ensure_connection(self):
         """Connect this medium if not already connected."""
-        if self._connected:
+        if self._real_medium is not None:
             return
         if self._vendor is None:
             vendor = ssh._get_ssh_vendor()
         else:
             vendor = self._vendor
-        self._ssh_connection = vendor.connect_ssh(self._username,
-                self._password, self._host, self._port,
-                command=[self._bzr_remote_path, 'serve', '--inet',
+        self._ssh_connection = vendor.connect_ssh(self._ssh_params.username,
+                self._ssh_params.password, self._ssh_params.host,
+                self._ssh_params.port,
+                command=[self._ssh_params.bzr_remote_path, 'serve', '--inet',
                          '--directory=/', '--allow-writes'])
-        self._read_from, self._write_to = \
-            self._ssh_connection.get_filelike_channels()
-        self._connected = True
+        io_kind, io_object = self._ssh_connection.get_sock_or_pipes()
+        if io_kind == 'socket':
+            self._real_medium = SmartClientAlreadyConnectedSocketMedium(
+                self.base, io_object)
+        elif io_kind == 'pipes':
+            read_from, write_to = io_object
+            self._real_medium = SmartSimplePipesClientMedium(
+                read_from, write_to, self.base)
+        else:
+            raise AssertionError(
+                "Unexpected io_kind %r from %r"
+                % (io_kind, self._ssh_connection))
 
     def _flush(self):
         """See SmartClientStreamMedium._flush()."""
-        self._write_to.flush()
+        self._real_medium._flush()
 
     def _read_bytes(self, count):
         """See SmartClientStreamMedium.read_bytes."""
-        if not self._connected:
+        if self._real_medium is None:
             raise errors.MediumNotConnected(self)
-        bytes_to_read = min(count, _MAX_READ_SIZE)
-        bytes = self._read_from.read(bytes_to_read)
-        self._report_activity(len(bytes), 'read')
-        return bytes
+        return self._real_medium.read_bytes(count)
 
 
 # Port 4155 is the default port for bzr://, registered with IANA.
@@ -832,22 +845,41 @@
 BZR_DEFAULT_PORT = 4155
 
 
-class SmartTCPClientMedium(SmartClientStreamMedium):
-    """A client medium using TCP."""
+class SmartClientSocketMedium(SmartClientStreamMedium):
+    """A client medium using a socket.
+    
+    This class isn't usable directly.  Use one of its subclasses instead.
+    """
 
-    def __init__(self, host, port, base):
-        """Creates a client that will connect on the first use."""
+    def __init__(self, base):
         SmartClientStreamMedium.__init__(self, base)
+        self._socket = None
         self._connected = False
-        self._host = host
-        self._port = port
-        self._socket = None
 
     def _accept_bytes(self, bytes):
         """See SmartClientMedium.accept_bytes."""
         self._ensure_connection()
         osutils.send_all(self._socket, bytes, self._report_activity)
 
+    def _ensure_connection(self):
+        """Connect this medium if not already connected."""
+        raise NotImplementedError(self._ensure_connection)
+
+    def _flush(self):
+        """See SmartClientStreamMedium._flush().
+
+        For sockets we do no flushing. For TCP sockets we may want to turn off
+        TCP_NODELAY and add a means to do a flush, but that can be done in the
+        future.
+        """
+
+    def _read_bytes(self, count):
+        """See SmartClientMedium.read_bytes."""
+        if not self._connected:
+            raise errors.MediumNotConnected(self)
+        return osutils.read_bytes_from_socket(
+            self._socket, self._report_activity)
+
     def disconnect(self):
         """See SmartClientMedium.disconnect()."""
         if not self._connected:
@@ -856,6 +888,16 @@
         self._socket = None
         self._connected = False
 
+
+class SmartTCPClientMedium(SmartClientSocketMedium):
+    """A client medium that creates a TCP connection."""
+
+    def __init__(self, host, port, base):
+        """Creates a client that will connect on the first use."""
+        SmartClientSocketMedium.__init__(self, base)
+        self._host = host
+        self._port = port
+
     def _ensure_connection(self):
         """Connect this medium if not already connected."""
         if self._connected:
@@ -895,19 +937,22 @@
                     (self._host, port, err_msg))
         self._connected = True
 
-    def _flush(self):
-        """See SmartClientStreamMedium._flush().
-
-        For TCP we do no flushing. We may want to turn off TCP_NODELAY and
-        add a means to do a flush, but that can be done in the future.
-        """
-
-    def _read_bytes(self, count):
-        """See SmartClientMedium.read_bytes."""
-        if not self._connected:
-            raise errors.MediumNotConnected(self)
-        return osutils.read_bytes_from_socket(
-            self._socket, self._report_activity)
+
+class SmartClientAlreadyConnectedSocketMedium(SmartClientSocketMedium):
+    """A client medium for an already connected socket.
+    
+    Note that this class will assume it "owns" the socket, so it will close it
+    when its disconnect method is called.
+    """
+
+    def __init__(self, base, sock):
+        SmartClientSocketMedium.__init__(self, base)
+        self._socket = sock
+        self._connected = True
+
+    def _ensure_connection(self):
+        # Already connected, by definition!  So nothing to do.
+        pass
 
 
 class SmartClientStreamMediumRequest(SmartClientMediumRequest):

=== modified file 'bzrlib/tests/blackbox/test_conflicts.py'
--- bzrlib/tests/blackbox/test_conflicts.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/blackbox/test_conflicts.py	2010-05-28 14:15:28 +0000
@@ -70,6 +70,12 @@
         out, err = self.run_bzr('conflicts --text')
         self.assertEqual(['my_other_file', 'myfile'], out.splitlines())
 
+    def test_conflicts_directory(self):
+        """Test --directory option"""
+        out, err = self.run_bzr('conflicts --directory a', working_dir='.')
+        self.assertEqual(3, len(out.splitlines()))
+        self.assertEqual('', err)
+
 
 class TestResolve(TestConflictsBase):
 
@@ -113,3 +119,10 @@
         self.build_tree_contents([('tree/file', 'a\n')])
         note = self.run_bzr('resolve', working_dir='tree')[1]
         self.assertContainsRe(note, 'All conflicts resolved.')
+
+    def test_resolve_all_directory(self):
+        """Test --directory option"""
+        out, err = self.run_bzr('resolve --all -d a', working_dir='.')
+        self.assertEqual('', err)
+        out, err = self.run_bzr('conflicts')
+        self.assertEqual(0, len(out.splitlines()))

=== modified file 'bzrlib/tests/blackbox/test_log.py'
--- bzrlib/tests/blackbox/test_log.py	2010-04-14 12:30:17 +0000
+++ bzrlib/tests/blackbox/test_log.py	2010-06-08 08:24:39 +0000
@@ -159,6 +159,14 @@
         self.assertLogRevnos(['-c1'], ['1'])
 
 
+class TestLogExcludeCommonAncestry(TestLogWithLogCatcher):
+
+    def test_exclude_common_ancestry_simple_revnos(self):
+        self.make_linear_branch()
+        self.assertLogRevnos(['-r1..3', '--exclude-common-ancestry'],
+                             ['3', '2'])
+
+
 class TestLogMergedLinearAncestry(TestLogWithLogCatcher):
 
     def setUp(self):
@@ -167,6 +175,18 @@
         # stop calling run_bzr, there is no point) --vila 100118.
         builder = branchbuilder.BranchBuilder(self.get_transport())
         builder.start_series()
+        # 1
+        # | \
+        # 2  1.1.1
+        # | / |
+        # 3  1.1.2
+        # |   |
+        # |  1.1.3
+        # | / |
+        # 4  1.1.4
+        # | /
+        # 5
+
         # mainline
         builder.build_snapshot('1', None, [
             ('add', ('', 'root-id', 'directory', ''))])

=== modified file 'bzrlib/tests/blackbox/test_merge_directive.py'
--- bzrlib/tests/blackbox/test_merge_directive.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_merge_directive.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -248,3 +248,15 @@
         tree1, tree2 = self.prepare_merge_directive()
         tree1.commit(u'messag\xe9')
         self.run_bzr('merge-directive ../tree2') # no exception raised
+
+    def test_merge_directive_directory(self):
+        """Test --directory option"""
+        import re
+        re_timestamp = re.compile(r'^# timestamp: .*', re.M)
+        self.prepare_merge_directive()
+        md1 = self.run_bzr('merge-directive ../tree2')[0]
+        md1 = re_timestamp.sub('# timestamp: XXX', md1)
+        os.chdir('..')
+        md2 = self.run_bzr('merge-directive --directory tree1 tree2')[0]
+        md2 = re_timestamp.sub('# timestamp: XXX', md2)
+        self.assertEqualDiff(md1.replace('../tree2', 'tree2'), md2)

=== modified file 'bzrlib/tests/blackbox/test_missing.py'
--- bzrlib/tests/blackbox/test_missing.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_missing.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2008 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -200,3 +200,23 @@
                           'Branches are up to date.\n' % location,
                           lines)
         self.assertEquals('', err)
+
+    def test_missing_directory(self):
+        """Test --directory option"""
+
+        # create a source branch
+        a_tree = self.make_branch_and_tree('a')
+        self.build_tree_contents([('a/a', 'initial\n')])
+        a_tree.add('a')
+        a_tree.commit(message='initial')
+
+        # clone and add a differing revision
+        b_tree = a_tree.bzrdir.sprout('b').open_workingtree()
+        self.build_tree_contents([('b/a', 'initial\nmore\n')])
+        b_tree.commit(message='more')
+        
+        out2, err2 = self.run_bzr('missing --directory a b', retcode=1)
+        os.chdir('a')
+        out1, err1 = self.run_bzr('missing ../b', retcode=1)
+        self.assertEqualDiff(out1, out2)
+        self.assertEqualDiff(err1, err2)

=== modified file 'bzrlib/tests/blackbox/test_remove_tree.py'
--- bzrlib/tests/blackbox/test_remove_tree.py	2009-11-16 05:57:00 +0000
+++ bzrlib/tests/blackbox/test_remove_tree.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 import os
 
+from bzrlib import shelf
 from bzrlib.tests.blackbox import ExternalBase
 
 
@@ -155,3 +156,22 @@
         self.run_bzr('remove-tree branch2 --force')
         self.failIfExists('branch2/foo')
         self.failIfExists('branch2/bar')
+
+    def test_remove_tree_shelved_changes(self):
+        # https://bugs.launchpad.net/bzr/+bug/586639
+        tree = self.make_branch_and_tree('.')
+        creator = shelf.ShelfCreator(tree, tree.basis_tree(), [])
+        self.addCleanup(creator.finalize)
+        shelf_id = tree.get_shelf_manager().shelve_changes(creator, 'Foo')
+        output = self.run_bzr_error(["Working tree .* has shelved changes"],
+                                    'remove-tree', retcode=3)
+
+    def test_remove_tree_shelved_changes_force(self):
+        tree = self.make_branch_and_tree('.')
+        creator = shelf.ShelfCreator(tree, tree.basis_tree(), [])
+        self.addCleanup(creator.finalize)
+        shelf_id = tree.get_shelf_manager().shelve_changes(creator, 'Foo')
+        self.run_bzr('remove-tree --force')
+        self.run_bzr('checkout')
+        # Ensure shelf is empty
+        self.assertIs(None, tree.get_shelf_manager().last_shelf())

=== modified file 'bzrlib/tests/blackbox/test_shelve.py'
--- bzrlib/tests/blackbox/test_shelve.py	2010-05-20 18:23:17 +0000
+++ bzrlib/tests/blackbox/test_shelve.py	2010-05-28 14:15:28 +0000
@@ -97,3 +97,18 @@
         tree.add('file')
         os.chdir('tree/dir')
         self.run_bzr('shelve --all ../file')
+
+
+class TestShelveUnshelve(TestCaseWithTransport):
+
+    def test_directory(self):
+        """Test --directory option"""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/a', 'initial\n')])
+        tree.add('a')
+        tree.commit(message='committed')
+        self.build_tree_contents([('tree/a', 'initial\nmore\n')])
+        self.run_bzr('shelve -d tree --all')
+        self.assertFileEqual('initial\n', 'tree/a')
+        self.run_bzr('unshelve --directory tree')
+        self.assertFileEqual('initial\nmore\n', 'tree/a')

=== modified file 'bzrlib/tests/blackbox/test_switch.py'
--- bzrlib/tests/blackbox/test_switch.py	2010-04-28 07:53:34 +0000
+++ bzrlib/tests/blackbox/test_switch.py	2010-05-28 14:15:28 +0000
@@ -252,3 +252,21 @@
         self.assertLength(0, calls)
         out, err = self.run_bzr('switch ../branch2')
         self.assertLength(1, calls)
+
+    def test_switch_lightweight_directory(self):
+        """Test --directory option"""
+
+        # create a source branch
+        a_tree = self.make_branch_and_tree('a')
+        self.build_tree_contents([('a/a', 'initial\n')])
+        a_tree.add('a')
+        a_tree.commit(message='initial')
+
+        # clone and add a differing revision
+        b_tree = a_tree.bzrdir.sprout('b').open_workingtree()
+        self.build_tree_contents([('b/a', 'initial\nmore\n')])
+        b_tree.commit(message='more')
+
+        self.run_bzr('checkout --lightweight a checkout')
+        self.run_bzr('switch --directory checkout b')
+        self.assertFileEqual('initial\nmore\n', 'checkout/a')

=== modified file 'bzrlib/tests/blackbox/test_whoami.py'
--- bzrlib/tests/blackbox/test_whoami.py	2010-04-27 17:09:48 +0000
+++ bzrlib/tests/blackbox/test_whoami.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,8 +20,10 @@
 import os
 
 import bzrlib
-from bzrlib import osutils
-from bzrlib.branch import Branch
+from bzrlib import (
+    osutils,
+    config,
+    )
 from bzrlib.tests.blackbox import ExternalBase
 
 
@@ -43,45 +45,29 @@
         b = bzrlib.branch.Branch.open('.')
         b.get_config().set_user_option('email',
                                        'Branch Identity <branch@identi.ty>')
-        bzr_email = os.environ.get('BZR_EMAIL')
-        if bzr_email is not None:
-            del os.environ['BZR_EMAIL']
-        try:
-            whoami = self.run_bzr("whoami")[0]
-            self.assertEquals('Branch Identity <branch@identi.ty>\n', whoami)
-            whoami_email = self.run_bzr("whoami --email")[0]
-            self.assertEquals('branch@identi.ty\n', whoami_email)
+        whoami = self.run_bzr("whoami")[0]
+        self.assertEquals('Branch Identity <branch@identi.ty>\n', whoami)
+        whoami_email = self.run_bzr("whoami --email")[0]
+        self.assertEquals('branch@identi.ty\n', whoami_email)
 
-            # Verify that the environment variable overrides the value
-            # in the file
-            os.environ['BZR_EMAIL'] = 'Different ID <other@environ.ment>'
-            whoami = self.run_bzr("whoami")[0]
-            self.assertEquals('Different ID <other@environ.ment>\n', whoami)
-            whoami_email = self.run_bzr("whoami --email")[0]
-            self.assertEquals('other@environ.ment\n', whoami_email)
-            del os.environ['BZR_EMAIL']
-        finally:
-            if bzr_email is not None:
-                os.environ['BZR_EMAIL'] = bzr_email
+        # Verify that the environment variable overrides the value
+        # in the file
+        os.environ['BZR_EMAIL'] = 'Different ID <other@environ.ment>'
+        whoami = self.run_bzr("whoami")[0]
+        self.assertEquals('Different ID <other@environ.ment>\n', whoami)
+        whoami_email = self.run_bzr("whoami --email")[0]
+        self.assertEquals('other@environ.ment\n', whoami_email)
 
     def test_whoami_utf8(self):
         """verify that an identity can be in utf-8."""
         wt = self.make_branch_and_tree('.')
         self.run_bzr(['whoami', u'Branch Identity \u20ac <branch@identi.ty>'],
                      encoding='utf-8')
-        bzr_email = os.environ.get('BZR_EMAIL')
-        if bzr_email is not None:
-            del os.environ['BZR_EMAIL']
-        try:
-            whoami = self.run_bzr("whoami", encoding='utf-8')[0]
-            self.assertEquals('Branch Identity \xe2\x82\xac ' +
-                              '<branch@identi.ty>\n', whoami)
-            whoami_email = self.run_bzr("whoami --email",
-                                        encoding='utf-8')[0]
-            self.assertEquals('branch@identi.ty\n', whoami_email)
-        finally:
-            if bzr_email is not None:
-                os.environ['BZR_EMAIL'] = bzr_email
+        whoami = self.run_bzr("whoami", encoding='utf-8')[0]
+        self.assertEquals('Branch Identity \xe2\x82\xac ' +
+                          '<branch@identi.ty>\n', whoami)
+        whoami_email = self.run_bzr("whoami --email", encoding='utf-8')[0]
+        self.assertEquals('branch@identi.ty\n', whoami_email)
 
     def test_whoami_ascii(self):
         """
@@ -92,18 +78,10 @@
         b = bzrlib.branch.Branch.open('.')
         b.get_config().set_user_option('email', u'Branch Identity \u20ac ' +
                                        '<branch@identi.ty>')
-        bzr_email = os.environ.get('BZR_EMAIL')
-        if bzr_email is not None:
-            del os.environ['BZR_EMAIL']
-        try:
-            whoami = self.run_bzr("whoami", encoding='ascii')[0]
-            self.assertEquals('Branch Identity ? <branch@identi.ty>\n', whoami)
-            whoami_email = self.run_bzr("whoami --email",
-                                        encoding='ascii')[0]
-            self.assertEquals('branch@identi.ty\n', whoami_email)
-        finally:
-            if bzr_email is not None:
-                os.environ['BZR_EMAIL'] = bzr_email
+        whoami = self.run_bzr("whoami", encoding='ascii')[0]
+        self.assertEquals('Branch Identity ? <branch@identi.ty>\n', whoami)
+        whoami_email = self.run_bzr("whoami --email", encoding='ascii')[0]
+        self.assertEquals('branch@identi.ty\n', whoami_email)
 
     def test_warning(self):
         """verify that a warning is displayed if no email is given."""
@@ -120,3 +98,39 @@
         osutils.set_or_unset_env('BZR_EMAIL', None)
         out, err = self.run_bzr(['whoami'], 3)
         self.assertContainsRe(err, 'Unable to determine your name')
+
+    def test_whoami_directory(self):
+        """Test --directory option."""
+        wt = self.make_branch_and_tree('subdir')
+        c = wt.branch.get_config()
+        c.set_user_option('email', 'Branch Identity <branch@identi.ty>')
+        out, err = self.run_bzr("whoami --directory subdir")
+        self.assertEquals('Branch Identity <branch@identi.ty>\n', out)
+        self.run_bzr(['whoami', '--directory', 'subdir', '--branch',
+                      'Changed Identity <changed@identi.ty>'])
+        self.assertEquals('Changed Identity <changed@identi.ty>',
+                          c.get_user_option('email'))
+
+    def test_whoami_remote_directory(self):
+        """Test --directory option with a remote directory."""
+        wt = self.make_branch_and_tree('subdir')
+        c = wt.branch.get_config()
+        c.set_user_option('email', 'Branch Identity <branch@identi.ty>')
+        url = self.get_readonly_url() + '/subdir'
+        out, err = self.run_bzr(['whoami', '--directory', url])
+        self.assertEquals('Branch Identity <branch@identi.ty>\n', out)
+        url = self.get_url('subdir')
+        self.run_bzr(['whoami', '--directory', url, '--branch',
+                      'Changed Identity <changed@identi.ty>'])
+        # The identity has been set in the branch config (but not the global
+        # config)
+        self.assertEquals('Changed Identity <changed@identi.ty>',
+                          c.get_user_option('email'))
+        global_conf = config.GlobalConfig()
+        self.assertEquals(None, global_conf.get_user_option('email'))
+
+    def test_whoami_nonbranch_directory(self):
+        """Test --directory mentioning a non-branch directory."""
+        wt = self.build_tree(['subdir/'])
+        out, err = self.run_bzr("whoami --directory subdir", retcode=3)
+        self.assertContainsRe(err, 'ERROR: Not a branch')

=== modified file 'bzrlib/tests/per_branch/test_locking.py'
--- bzrlib/tests/per_branch/test_locking.py	2010-05-20 03:20:04 +0000
+++ bzrlib/tests/per_branch/test_locking.py	2010-06-11 07:59:43 +0000
@@ -533,6 +533,7 @@
     def test_lock_write_raises_in_lock_read(self):
         branch = self.make_branch('b')
         branch.lock_read()
+        self.addCleanup(branch.unlock)
         err = self.assertRaises(errors.ReadOnlyError, branch.lock_write)
 
     def test_lock_and_unlock_leaves_repo_unlocked(self):

=== modified file 'bzrlib/tests/per_branch/test_pull.py'
--- bzrlib/tests/per_branch/test_pull.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_pull.py	2010-06-14 06:37:11 +0000
@@ -98,17 +98,6 @@
                           master_tree.branch.pull, other.branch, local = True)
         self.assertEqual([rev1], master_tree.branch.revision_history())
 
-    def test_pull_raises_specific_error_on_master_connection_error(self):
-        master_tree = self.make_branch_and_tree('master')
-        checkout = master_tree.branch.create_checkout('checkout')
-        other = master_tree.branch.bzrdir.sprout('other').open_workingtree()
-        # move the branch out of the way on disk to cause a connection
-        # error.
-        os.rename('master', 'master_gone')
-        # try to pull, which should raise a BoundBranchConnectionFailure.
-        self.assertRaises(errors.BoundBranchConnectionFailure,
-                checkout.branch.pull, other.branch)
-
     def test_pull_returns_result(self):
         parent = self.make_branch_and_tree('parent')
         parent.commit('1st post', rev_id='P1')

=== modified file 'bzrlib/tests/per_interbranch/__init__.py'
--- bzrlib/tests/per_interbranch/__init__.py	2009-05-07 05:08:46 +0000
+++ bzrlib/tests/per_interbranch/__init__.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -76,11 +76,9 @@
     # test the default InterBranch between format 6 and the current
     # default format.
     for optimiser_class in InterBranch._optimisers:
-        format_from_test, format_to_test = \
-            optimiser_class._get_branch_formats_to_test()
-        if format_to_test is not None:
-            result.append((optimiser_class,
-                           format_from_test, format_to_test))
+        for format_from_test, format_to_test in \
+            optimiser_class._get_branch_formats_to_test():
+            result.append((optimiser_class, format_from_test, format_to_test))
     # if there are specific combinations we want to use, we can add them
     # here.
     return result
@@ -144,8 +142,38 @@
         return newbranch.bzrdir
 
 
+class StubWithFormat(object):
+    """A stub object used to check that convenience methods call Inter's."""
+
+    _format = object()
+
+
+class StubMatchingInter(object):
+    """An inter for tests.
+
+    This is not a subclass of InterBranch so that missing methods are caught
+    and added rather than actually trying to do something.
+    """
+
+    _uses = []
+
+    def __init__(self, source, target):
+        self.source = source
+        self.target = target
+
+    @classmethod
+    def is_compatible(klass, source, target):
+        return StubWithFormat._format in (source._format, target._format)
+
+    def copy_content_into(self, *args, **kwargs):
+        self.__class__._uses.append(
+            (self, 'copy_content_into', args, kwargs))
+
+
 def load_tests(standard_tests, module, loader):
     submod_tests = loader.loadTestsFromModuleNames([
+        'bzrlib.tests.per_interbranch.test_get',
+        'bzrlib.tests.per_interbranch.test_copy_content_into',
         'bzrlib.tests.per_interbranch.test_pull',
         'bzrlib.tests.per_interbranch.test_push',
         'bzrlib.tests.per_interbranch.test_update_revisions',

=== added file 'bzrlib/tests/per_interbranch/test_copy_content_into.py'
--- bzrlib/tests/per_interbranch/test_copy_content_into.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_interbranch/test_copy_content_into.py	2010-06-17 06:30:22 +0000
@@ -0,0 +1,47 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for bzrlib.branch.InterBranch.copy_content_into."""
+
+from bzrlib import branch
+from bzrlib.tests.per_interbranch import (
+    StubMatchingInter,
+    StubWithFormat,
+    TestCaseWithInterBranch,
+    )
+
+
+class TestCopyContentInto(TestCaseWithInterBranch):
+
+    def test_contract_convenience_method(self):
+        self.tree1 = self.make_from_branch_and_tree('tree1')
+        rev1 = self.tree1.commit('one')
+        branch2 = self.make_to_branch('tree2')
+        branch2.repository.fetch(self.tree1.branch.repository)
+        self.tree1.branch.copy_content_into(branch2, revision_id=rev1)
+
+    def test_inter_is_used(self):
+        self.tree1 = self.make_from_branch_and_tree('tree1')
+        self.addCleanup(branch.InterBranch.unregister_optimiser,
+            StubMatchingInter)
+        branch.InterBranch.register_optimiser(StubMatchingInter)
+        del StubMatchingInter._uses[:]
+        self.tree1.branch.copy_content_into(StubWithFormat(), revision_id='54')
+        self.assertLength(1, StubMatchingInter._uses)
+        use = StubMatchingInter._uses[0]
+        self.assertEqual('copy_content_into', use[1])
+        self.assertEqual('54', use[3]['revision_id'])
+        del StubMatchingInter._uses[:]

=== added file 'bzrlib/tests/per_interbranch/test_get.py'
--- bzrlib/tests/per_interbranch/test_get.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_interbranch/test_get.py	2010-06-17 06:30:22 +0000
@@ -0,0 +1,34 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for bzrlib.branch.InterBranch.get."""
+
+from bzrlib import (
+    branch,
+    )
+from bzrlib.tests.per_interbranch import (
+    TestCaseWithInterBranch,
+    )
+
+
+class TestInterBranchGet(TestCaseWithInterBranch):
+
+    def test_gets_right_inter(self):
+        self.tree1 = self.make_from_branch_and_tree('tree1')
+        branch2 = self.make_to_branch('tree2')
+        self.assertIs(branch.InterBranch.get(
+            self.tree1.branch, branch2).__class__,
+            self.interbranch_class)

=== modified file 'bzrlib/tests/per_interbranch/test_pull.py'
--- bzrlib/tests/per_interbranch/test_pull.py	2009-07-10 05:49:34 +0000
+++ bzrlib/tests/per_interbranch/test_pull.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2007, 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -76,13 +76,13 @@
     def test_pull_raises_specific_error_on_master_connection_error(self):
         master_tree = self.make_from_branch_and_tree('master')
         checkout = master_tree.branch.create_checkout('checkout')
-        other = self.sprout_to(master_tree.branch.bzrdir, 'other').open_workingtree()
+        other = self.sprout_to(master_tree.branch.bzrdir, 'other').open_branch()
         # move the branch out of the way on disk to cause a connection
         # error.
         os.rename('master', 'master_gone')
         # try to pull, which should raise a BoundBranchConnectionFailure.
         self.assertRaises(errors.BoundBranchConnectionFailure,
-                checkout.branch.pull, other.branch)
+                checkout.branch.pull, other)
 
     def test_pull_returns_result(self):
         parent = self.make_from_branch_and_tree('parent')

=== modified file 'bzrlib/tests/per_interbranch/test_push.py'
--- bzrlib/tests/per_interbranch/test_push.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_interbranch/test_push.py	2010-06-17 06:30:22 +0000
@@ -145,7 +145,7 @@
         except (errors.IncompatibleFormat, errors.UninitializableFormat):
             # This Branch format cannot create shared repositories
             return
-        # This is a little bit trickier because make_branch_and_tree will not
+        # This is a little bit trickier because make_from_branch_and_tree will not
         # re-use a shared repository.
         try:
             a_branch = self.make_from_branch('repo/tree')

=== modified file 'bzrlib/tests/per_interbranch/test_update_revisions.py'
--- bzrlib/tests/per_interbranch/test_update_revisions.py	2009-05-07 05:08:46 +0000
+++ bzrlib/tests/per_interbranch/test_update_revisions.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 
     def setUp(self):
         super(TestUpdateRevisions, self).setUp()
-        self.tree1 = self.make_branch_and_tree('tree1')
+        self.tree1 = self.make_from_branch_and_tree('tree1')
         rev1 = self.tree1.commit('one')
         branch2 = self.make_to_branch('tree2')
         branch2.repository.fetch(self.tree1.branch.repository)

=== modified file 'bzrlib/tests/script.py'
--- bzrlib/tests/script.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/script.py	2010-06-11 06:33:15 +0000
@@ -24,6 +24,7 @@
 import glob
 import os
 import shlex
+import textwrap
 from cStringIO import StringIO
 
 from bzrlib import (
@@ -73,6 +74,7 @@
     cmd_line = 1
     lineno = 0
     input, output, error = None, None, None
+    text = textwrap.dedent(text)
     for line in text.split('\n'):
         lineno += 1
         # Keep a copy for error reporting
@@ -475,3 +477,7 @@
     def run_command(self, cmd, input, output, error):
         return self.script_runner.run_command(self, cmd, input, output, error)
 
+
+def run_script(test_case, script_string):
+    """Run the given script within a testcase"""
+    return ScriptRunner().run_script(test_case, script_string)

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2010-05-14 09:34:16 +0000
+++ bzrlib/tests/stub_sftp.py	2010-06-02 04:50:35 +0000
@@ -135,7 +135,7 @@
         try:
             out = [ ]
             # TODO: win32 incorrectly lists paths with non-ascii if path is not
-            # unicode. However on Linux the server should only deal with
+            # unicode. However on unix the server should only deal with
             # bytestreams and posix.listdir does the right thing
             if sys.platform == 'win32':
                 flist = [f.encode('utf8') for f in os.listdir(path)]
@@ -452,7 +452,7 @@
             # Normalize the path or it will be wrongly escaped
             self._homedir = osutils.normpath(self._homedir)
         else:
-            # But Linux SFTP servers should just deal in bytestreams
+            # But unix SFTP servers should just deal in bytestreams
             self._homedir = os.getcwd()
         if self._server_homedir is None:
             self._server_homedir = self._homedir

=== modified file 'bzrlib/tests/test__dirstate_helpers.py'
--- bzrlib/tests/test__dirstate_helpers.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test__dirstate_helpers.py	2010-06-02 04:50:35 +0000
@@ -737,7 +737,7 @@
 
     def test_trailing_garbage(self):
         tree, state, expected = self.create_basic_dirstate()
-        # On Linux, we can write extra data as long as we haven't read yet, but
+        # On Unix, we can write extra data as long as we haven't read yet, but
         # on Win32, if you've opened the file with FILE_SHARE_READ, trying to
         # open it in append mode will fail.
         state.unlock()

=== modified file 'bzrlib/tests/test_cleanup.py'
--- bzrlib/tests/test_cleanup.py	2010-03-03 22:59:21 +0000
+++ bzrlib/tests/test_cleanup.py	2010-04-27 07:29:11 +0000
@@ -20,6 +20,7 @@
 from bzrlib.cleanup import (
     _do_with_cleanups,
     _run_cleanup,
+    ObjectWithCleanups,
     OperationWithCleanups,
     )
 from bzrlib.tests import TestCase
@@ -276,3 +277,17 @@
             [('func called', 'foo'), 'cleanup 1', 'cleanup 2', 'cleanup 3',
             'cleanup 4'], call_log)
 
+
+class SampleWithCleanups(ObjectWithCleanups):
+
+    pass
+
+
+class TestObjectWithCleanups(TestCase):
+
+    def test_object_with_cleanups(self):
+        a = []
+        s = SampleWithCleanups()
+        s.add_cleanup(a.append, 42)
+        s.cleanup_now()
+        self.assertEqual(a, [42])

=== modified file 'bzrlib/tests/test_directory_service.py'
--- bzrlib/tests/test_directory_service.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_directory_service.py	2010-06-02 04:50:35 +0000
@@ -28,7 +28,7 @@
 class FooService(object):
     """A directory service that maps the name to a FILE url"""
 
-    # eg 'file:///foo' on Linux, or 'file:///C:/foo' on Windows
+    # eg 'file:///foo' on Unix, or 'file:///C:/foo' on Windows
     base = urlutils.local_path_to_url('/foo')
 
     def look_up(self, name, url):

=== modified file 'bzrlib/tests/test_import_tariff.py'
--- bzrlib/tests/test_import_tariff.py	2010-05-31 01:28:29 +0000
+++ bzrlib/tests/test_import_tariff.py	2010-06-04 03:09:35 +0000
@@ -93,9 +93,12 @@
             'bzrlib.bundle.commands',
             'bzrlib.cmd_version_info',
             'bzrlib.foreign',
+            'bzrlib.merge3',
+            'bzrlib.patiencediff',
             'bzrlib.remote',
             'bzrlib.sign_my_commits',
             'bzrlib.smart',
+            'bzrlib.transform',
             'kerberos',
             'smtplib',
             'tarfile',

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_knit.py	2010-06-04 03:09:35 +0000
@@ -17,30 +17,25 @@
 """Tests for Knit data structure"""
 
 from cStringIO import StringIO
-import difflib
-import gzip
 import sys
 
 from bzrlib import (
     errors,
-    generate_ids,
     knit,
     multiparent,
     osutils,
     pack,
     tests,
+    tuned_gzip,
     )
 from bzrlib.errors import (
-    RevisionAlreadyPresent,
     KnitHeaderError,
-    RevisionNotPresent,
     NoSuchFile,
     )
 from bzrlib.index import *
 from bzrlib.knit import (
     AnnotatedKnitContent,
     KnitContent,
-    KnitSequenceMatcher,
     KnitVersionedFiles,
     PlainKnitContent,
     _VFContentMapGenerator,
@@ -50,18 +45,15 @@
     _KnitKeyAccess,
     make_file_factory,
     )
+from bzrlib.patiencediff import PatienceSequenceMatcher
 from bzrlib.repofmt import pack_repo
 from bzrlib.tests import (
-    Feature,
-    KnownFailure,
     TestCase,
     TestCaseWithMemoryTransport,
     TestCaseWithTransport,
     TestNotApplicable,
     )
 from bzrlib.transport import get_transport
-from bzrlib.transport.memory import MemoryTransport
-from bzrlib.tuned_gzip import GzipFile
 from bzrlib.versionedfile import (
     AbsentContentFactory,
     ConstantMapper,
@@ -106,8 +98,8 @@
         line_delta = source_content.line_delta(target_content)
         delta_blocks = list(KnitContent.get_line_delta_blocks(line_delta,
             source_lines, target_lines))
-        matcher = KnitSequenceMatcher(None, source_lines, target_lines)
-        matcher_blocks = list(list(matcher.get_matching_blocks()))
+        matcher = PatienceSequenceMatcher(None, source_lines, target_lines)
+        matcher_blocks = list(matcher.get_matching_blocks())
         self.assertEqual(matcher_blocks, delta_blocks)
 
     def test_get_line_delta_blocks(self):
@@ -700,7 +692,7 @@
 
     def create_gz_content(self, text):
         sio = StringIO()
-        gz_file = gzip.GzipFile(mode='wb', fileobj=sio)
+        gz_file = tuned_gzip.GzipFile(mode='wb', fileobj=sio)
         gz_file.write(text)
         gz_file.close()
         return sio.getvalue()

=== modified file 'bzrlib/tests/test_log.py'
--- bzrlib/tests/test_log.py	2010-05-05 08:18:32 +0000
+++ bzrlib/tests/test_log.py	2010-06-08 09:50:27 +0000
@@ -1724,13 +1724,13 @@
         return br
 
     def assertLogRevnos(self, expected_revnos, b, start, end,
-                        exclude_common_ancestry):
+                        exclude_common_ancestry, generate_merge_revisions=True):
         # FIXME: the layering in log makes it hard to test intermediate levels,
         # I wish adding filters with their parameters were easier...
         # -- vila 20100413
         iter_revs = log._calc_view_revisions(
             b, start, end, direction='reverse',
-            generate_merge_revisions=True,
+            generate_merge_revisions=generate_merge_revisions,
             exclude_common_ancestry=exclude_common_ancestry)
         self.assertEqual(expected_revnos,
                          [revid for revid, revno, depth in iter_revs])
@@ -1738,11 +1738,19 @@
     def test_merge_sorted_exclude_ancestry(self):
         b = self.make_branch_with_alternate_ancestries()
         self.assertLogRevnos(['3', '1.1.2', '1.2.1', '1.1.1', '2', '1'],
-                             b, '1', '3', False)
+                             b, '1', '3', exclude_common_ancestry=False)
         # '2' is part of the '3' ancestry but not part of '1.1.1' ancestry so
         # it should be mentioned even if merge_sort order will make it appear
         # after 1.1.1
         self.assertLogRevnos(['3', '1.1.2', '1.2.1', '2'],
-                             b, '1.1.1', '3', True)
+                             b, '1.1.1', '3', exclude_common_ancestry=True)
 
+    def test_merge_sorted_simple_revnos_exclude_ancestry(self):
+        b = self.make_branch_with_alternate_ancestries()
+        self.assertLogRevnos(['3', '2'],
+                             b, '1', '3', exclude_common_ancestry=True,
+                             generate_merge_revisions=False)
+        self.assertLogRevnos(['3', '1.1.2', '1.2.1', '1.1.1', '2'],
+                             b, '1', '3', exclude_common_ancestry=True,
+                             generate_merge_revisions=True)
 

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2010-05-16 13:49:19 +0000
+++ bzrlib/tests/test_plugins.py	2010-06-11 08:02:42 +0000
@@ -27,6 +27,7 @@
 
 import bzrlib
 from bzrlib import (
+    errors,
     osutils,
     plugin,
     plugins,
@@ -79,6 +80,16 @@
         if getattr(bzrlib.plugins, name, None) is not None:
             delattr(bzrlib.plugins, name)
 
+    def _unregister_plugin_submodule(self, plugin_name, submodule_name):
+        """Remove the submodule from sys.modules and the bzrlib namespace."""
+        py_name = 'bzrlib.plugins.%s.%s' % (plugin_name, submodule_name)
+        if py_name in sys.modules:
+            del sys.modules[py_name]
+        plugin = getattr(bzrlib.plugins, plugin_name, None)
+        if plugin is not None:
+            if getattr(plugin, submodule_name, None) is not None:
+                delattr(plugin, submodule_name)
+
     def assertPluginUnknown(self, name):
         self.failIf(getattr(bzrlib.plugins, name, None) is not None)
         self.failIf('bzrlib.plugins.%s' % name in sys.modules)
@@ -798,14 +809,35 @@
         self.assertLength(0, self.warnings)
 
 
+class TestLoadPluginAtSyntax(tests.TestCase):
+
+    def _get_paths(self, paths):
+        return plugin._get_specific_plugin_paths(paths)
+
+    def test_empty(self):
+        self.assertEquals([], self._get_paths(None))
+        self.assertEquals([], self._get_paths(''))
+
+    def test_one_path(self):
+        self.assertEquals([('b', 'man')], self._get_paths('b@man'))
+
+    def test_bogus_path(self):
+        # We need a '@'
+        self.assertRaises(errors.BzrCommandError, self._get_paths, 'batman')
+        # Too much '@' isn't good either
+        self.assertRaises(errors.BzrCommandError, self._get_paths,
+                          'batman@mobile@cave')
+        # An empty description probably indicates a problem
+        self.assertRaises(errors.BzrCommandError, self._get_paths,
+                          os.pathsep.join(['batman@cave', '', 'robin@mobile']))
+
+
 class TestLoadPluginAt(tests.TestCaseInTempDir, TestPluginMixin):
 
     def setUp(self):
         super(TestLoadPluginAt, self).setUp()
         # Make sure we don't pollute the plugins namespace
         self.overrideAttr(plugins, '__path__')
-        # Be paranoid in case a test fail
-        self.addCleanup(self._unregister_plugin, 'test_foo')
         # Reset the flag that protect against double loading
         self.overrideAttr(plugin, '_loaded', False)
         # Create the same plugin in two directories
@@ -813,6 +845,8 @@
         # The "normal" directory, we use 'standard' instead of 'plugins' to
         # avoid depending on the precise naming.
         self.create_plugin_package('test_foo', dir='standard/test_foo')
+        # All the tests will load the 'test_foo' plugin from various locations
+        self.addCleanup(self._unregister_plugin, 'test_foo')
 
     def assertTestFooLoadedFrom(self, path):
         self.assertPluginKnown('test_foo')
@@ -860,6 +894,8 @@
     def test_submodule_loading(self):
         # We create an additional directory under the one for test_foo
         self.create_plugin_package('test_bar', dir='non-standard-dir/test_bar')
+        self.addCleanup(self._unregister_plugin_submodule,
+                        'test_foo', 'test_bar')
         osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.set_plugins_path(['standard'])
         import bzrlib.plugins.test_foo
@@ -869,6 +905,22 @@
         self.assertIsSameRealPath('non-standard-dir/test_bar/__init__.py',
                                   bzrlib.plugins.test_foo.test_bar.__file__)
 
+    def test_relative_submodule_loading(self):
+        self.create_plugin_package('test_foo', dir='another-dir', source='''
+import test_bar
+''')
+        # We create an additional directory under the one for test_foo
+        self.create_plugin_package('test_bar', dir='another-dir/test_bar')
+        self.addCleanup(self._unregister_plugin_submodule,
+                        'test_foo', 'test_bar')
+        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@another-dir')
+        plugin.set_plugins_path(['standard'])
+        import bzrlib.plugins.test_foo
+        self.assertEqual('bzrlib.plugins.test_foo',
+                         bzrlib.plugins.test_foo.__package__)
+        self.assertIsSameRealPath('another-dir/test_bar/__init__.py',
+                                  bzrlib.plugins.test_foo.test_bar.__file__)
+
     def test_loading_from___init__only(self):
         # We rename the existing __init__.py file to ensure that we don't load
         # a random file

=== modified file 'bzrlib/tests/test_script.py'
--- bzrlib/tests/test_script.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_script.py	2010-06-11 06:33:15 +0000
@@ -51,6 +51,18 @@
             [(['cat', '>file'], 'content\n', None, None)],
             script._script_to_commands('$ cat >file\n<content\n'))
 
+    def test_indented(self):
+        # scripts are commonly given indented within the test source code, and
+        # common indentation is stripped off
+        story = """
+            $ bzr add
+            adding file
+            adding file2
+            """
+        self.assertEquals([(['bzr', 'add'], None,
+                            'adding file\nadding file2\n', None)],
+                          script._script_to_commands(story))
+
     def test_command_with_output(self):
         story = """
 $ bzr add

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2010-03-18 23:11:15 +0000
+++ bzrlib/tests/test_smart_transport.py	2010-06-16 05:47:02 +0000
@@ -46,6 +46,7 @@
         local,
         memory,
         remote,
+        ssh,
         )
 
 
@@ -63,7 +64,7 @@
         return StringIOSSHConnection(self)
 
 
-class StringIOSSHConnection(object):
+class StringIOSSHConnection(ssh.SSHConnection):
     """A SSH connection that uses StringIO to buffer writes and answer reads."""
 
     def __init__(self, vendor):
@@ -71,9 +72,11 @@
 
     def close(self):
         self.vendor.calls.append(('close', ))
+        self.vendor.read_from.close()
+        self.vendor.write_to.close()
 
-    def get_filelike_channels(self):
-        return self.vendor.read_from, self.vendor.write_to
+    def get_sock_or_pipes(self):
+        return 'pipes', (self.vendor.read_from, self.vendor.write_to)
 
 
 class _InvalidHostnameFeature(tests.Feature):
@@ -243,9 +246,9 @@
         unopened_port = sock.getsockname()[1]
         # having vendor be invalid means that if it tries to connect via the
         # vendor it will blow up.
-        client_medium = medium.SmartSSHClientMedium('127.0.0.1', unopened_port,
-            username=None, password=None, base='base', vendor="not a vendor",
-            bzr_remote_path='bzr')
+        ssh_params = medium.SSHParams('127.0.0.1', unopened_port, None, None)
+        client_medium = medium.SmartSSHClientMedium(
+            'base', ssh_params, "not a vendor")
         sock.close()
 
     def test_ssh_client_connects_on_first_use(self):
@@ -253,9 +256,9 @@
         # it bytes.
         output = StringIO()
         vendor = StringIOSSHVendor(StringIO(), output)
-        client_medium = medium.SmartSSHClientMedium(
-            'a hostname', 'a port', 'a username', 'a password', 'base', vendor,
-            'bzr')
+        ssh_params = medium.SSHParams(
+            'a hostname', 'a port', 'a username', 'a password', 'bzr')
+        client_medium = medium.SmartSSHClientMedium('base', ssh_params, vendor)
         client_medium._accept_bytes('abc')
         self.assertEqual('abc', output.getvalue())
         self.assertEqual([('connect_ssh', 'a username', 'a password',
@@ -268,8 +271,10 @@
         # it bytes.
         output = StringIO()
         vendor = StringIOSSHVendor(StringIO(), output)
-        client_medium = medium.SmartSSHClientMedium('a hostname', 'a port',
-            'a username', 'a password', 'base', vendor, bzr_remote_path='fugly')
+        ssh_params = medium.SSHParams(
+            'a hostname', 'a port', 'a username', 'a password',
+            bzr_remote_path='fugly')
+        client_medium = medium.SmartSSHClientMedium('base', ssh_params, vendor)
         client_medium._accept_bytes('abc')
         self.assertEqual('abc', output.getvalue())
         self.assertEqual([('connect_ssh', 'a username', 'a password',
@@ -284,7 +289,7 @@
         output = StringIO()
         vendor = StringIOSSHVendor(input, output)
         client_medium = medium.SmartSSHClientMedium(
-            'a hostname', base='base', vendor=vendor, bzr_remote_path='bzr')
+            'base', medium.SSHParams('a hostname'), vendor)
         client_medium._accept_bytes('abc')
         client_medium.disconnect()
         self.assertTrue(input.closed)
@@ -305,7 +310,7 @@
         output = StringIO()
         vendor = StringIOSSHVendor(input, output)
         client_medium = medium.SmartSSHClientMedium(
-            'a hostname', base='base', vendor=vendor, bzr_remote_path='bzr')
+            'base', medium.SSHParams('a hostname'), vendor)
         client_medium._accept_bytes('abc')
         client_medium.disconnect()
         # the disconnect has closed output, so we need a new output for the
@@ -334,14 +339,14 @@
         # Doing a disconnect on a new (and thus unconnected) SSH medium
         # does not fail.  It's ok to disconnect an unconnected medium.
         client_medium = medium.SmartSSHClientMedium(
-            None, base='base', bzr_remote_path='bzr')
+            'base', medium.SSHParams(None))
         client_medium.disconnect()
 
     def test_ssh_client_raises_on_read_when_not_connected(self):
         # Doing a read on a new (and thus unconnected) SSH medium raises
         # MediumNotConnected.
         client_medium = medium.SmartSSHClientMedium(
-            None, base='base', bzr_remote_path='bzr')
+            'base', medium.SSHParams(None))
         self.assertRaises(errors.MediumNotConnected, client_medium.read_bytes,
                           0)
         self.assertRaises(errors.MediumNotConnected, client_medium.read_bytes,
@@ -359,7 +364,7 @@
         output.flush = logging_flush
         vendor = StringIOSSHVendor(input, output)
         client_medium = medium.SmartSSHClientMedium(
-            'a hostname', base='base', vendor=vendor, bzr_remote_path='bzr')
+            'base', medium.SSHParams('a hostname'), vendor=vendor)
         # this call is here to ensure we only flush once, not on every
         # _accept_bytes call.
         client_medium._accept_bytes('abc')

=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2010-05-27 22:10:42 +0000
+++ bzrlib/tests/test_urlutils.py	2010-06-15 03:08:31 +0000
@@ -156,7 +156,7 @@
         # Weird stuff
         # Can't have slashes or colons in the scheme
         test_one('/path/to/://foo', None)
-        test_one('path:path://foo', None)
+        test_one('scheme:stuff://foo', ('scheme', 'stuff://foo'))
         # Must have more than one character for scheme
         test_one('C://foo', None)
         test_one('ab://foo', ('ab', 'foo'))
@@ -195,6 +195,20 @@
             dirname('path/to/foo/', exclude_trailing_slash=False))
         self.assertEqual('path/..', dirname('path/../foo'))
         self.assertEqual('../path', dirname('../path/foo'))
+    
+    def test_is_url(self):
+        self.assertTrue(urlutils.is_url('http://foo/bar'))
+        self.assertTrue(urlutils.is_url('bzr+ssh://foo/bar'))
+        self.assertTrue(urlutils.is_url('lp:foo/bar'))
+        self.assertTrue(urlutils.is_url('file:///foo/bar'))
+        self.assertFalse(urlutils.is_url(''))
+        self.assertFalse(urlutils.is_url('foo'))
+        self.assertFalse(urlutils.is_url('foo/bar'))
+        self.assertFalse(urlutils.is_url('/foo'))
+        self.assertFalse(urlutils.is_url('/foo/bar'))
+        self.assertFalse(urlutils.is_url('C:/'))
+        self.assertFalse(urlutils.is_url('C:/foo'))
+        self.assertFalse(urlutils.is_url('C:/foo/bar'))
 
     def test_join(self):
         def test(expected, *args):
@@ -210,6 +224,8 @@
         test('http://foo/bar/baz', 'http://foo', 'bar/baz')
         test('http://foo/baz', 'http://foo', 'bar/../baz')
         test('http://foo/baz', 'http://foo/bar/', '../baz')
+        test('lp:foo/bar', 'lp:foo', 'bar')
+        test('lp:foo/bar/baz', 'lp:foo', 'bar/baz')
 
         # Absolute paths
         test('http://foo', 'http://foo') # abs url with nothing is preserved.
@@ -219,6 +235,9 @@
         test('http://bar/', 'http://foo', 'http://bar/')
         test('http://bar/a', 'http://foo', 'http://bar/a')
         test('http://bar/a/', 'http://foo', 'http://bar/a/')
+        test('lp:bar', 'http://foo', 'lp:bar')
+        test('lp:bar', 'lp:foo', 'lp:bar')
+        test('file:///stuff', 'lp:foo', 'file:///stuff')
 
         # From a base path
         test('file:///foo', 'file:///', 'foo')

=== modified file 'bzrlib/textmerge.py'
--- bzrlib/textmerge.py	2009-03-23 14:59:43 +0000
+++ bzrlib/textmerge.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,10 @@
 #         Aaron Bentley <aaron.bentley@utoronto.ca>
 
 
-import bzrlib.patiencediff
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import patiencediff
+""")
 
 
 class TextMerge(object):
@@ -131,7 +134,8 @@
         """Return structured merge info.
         See TextMerge docstring.
         """
-        sm = bzrlib.patiencediff.PatienceSequenceMatcher(None, self.lines_a, self.lines_b)
+        sm = patiencediff.PatienceSequenceMatcher(
+            None, self.lines_a, self.lines_b)
         pos_a = 0
         pos_b = 0
         for ai, bi, l in sm.get_matching_blocks():

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2010-05-25 17:27:52 +0000
+++ bzrlib/transform.py	2010-05-29 15:40:14 +0000
@@ -2291,9 +2291,6 @@
     for num, _unused in enumerate(wt.all_file_ids()):
         if num > 0:  # more than just a root
             raise errors.WorkingTreeAlreadyPopulated(base=wt.basedir)
-    existing_files = set()
-    for dir, files in wt.walkdirs():
-        existing_files.update(f[0] for f in files)
     file_trans_id = {}
     top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
     pp = ProgressPhase("Build phase", 2, top_pb)
@@ -2323,6 +2320,15 @@
                 precomputed_delta = []
             else:
                 precomputed_delta = None
+            # Check if tree inventory has content. If so, we populate
+            # existing_files with the directory content. If there are no
+            # entries we skip populating existing_files as its not used.
+            # This improves performance and unncessary work on large
+            # directory trees. (#501307)
+            if total > 0:
+                existing_files = set()
+                for dir, files in wt.walkdirs():
+                    existing_files.update(f[0] for f in files)
             for num, (tree_path, entry) in \
                 enumerate(tree.inventory.iter_entries_by_dir()):
                 pb.update("Building tree", num - len(deferred_contents), total)

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2010-05-30 16:26:11 +0000
+++ bzrlib/transport/__init__.py	2010-06-02 01:45:31 +0000
@@ -1551,9 +1551,6 @@
         return transport
 
 
-# We try to recognize an url lazily (ignoring user, password, etc)
-_urlRE = re.compile(r'^(?P<proto>[^:/\\]+)://(?P<rest>.*)$')
-
 def get_transport(base, possible_transports=None):
     """Open a transport to access a URL or directory.
 
@@ -1572,8 +1569,7 @@
     base = directories.dereference(base)
 
     def convert_path_to_url(base, error_str):
-        m = _urlRE.match(base)
-        if m:
+        if urlutils.is_url(base):
             # This looks like a URL, but we weren't able to
             # instantiate it as such raise an appropriate error
             # FIXME: we have a 'error_str' unused and we use last_err below

=== modified file 'bzrlib/transport/remote.py'
--- bzrlib/transport/remote.py	2010-02-09 20:28:26 +0000
+++ bzrlib/transport/remote.py	2010-06-17 09:23:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -514,9 +514,9 @@
         if user is None:
             auth = config.AuthenticationConfig()
             user = auth.get_user('ssh', self._host, self._port)
-        client_medium = medium.SmartSSHClientMedium(self._host, self._port,
-            user, self._password, self.base,
-            bzr_remote_path=bzr_remote_path)
+        ssh_params = medium.SSHParams(self._host, self._port, user,
+            self._password, bzr_remote_path)
+        client_medium = medium.SmartSSHClientMedium(self.base, ssh_params)
         return client_medium, (user, self._password)
 
 

=== modified file 'bzrlib/transport/ssh.py'
--- bzrlib/transport/ssh.py	2010-03-02 06:41:36 +0000
+++ bzrlib/transport/ssh.py	2010-06-16 06:21:34 +0000
@@ -239,8 +239,7 @@
     def connect_ssh(self, username, password, host, port, command):
         """Make an SSH connection.
 
-        :returns: something with a `close` method, and a `get_filelike_channels`
-            method that returns a pair of (read, write) filelike objects.
+        :returns: an SSHConnection.
         """
         raise NotImplementedError(self.connect_ssh)
 
@@ -269,17 +268,6 @@
 register_ssh_vendor('loopback', LoopbackVendor())
 
 
-class _ParamikoSSHConnection(object):
-    def __init__(self, channel):
-        self.channel = channel
-
-    def get_filelike_channels(self):
-        return self.channel.makefile('rb'), self.channel.makefile('wb')
-
-    def close(self):
-        return self.channel.close()
-
-
 class ParamikoVendor(SSHVendor):
     """Vendor that uses paramiko."""
 
@@ -363,11 +351,23 @@
     """Abstract base class for vendors that use pipes to a subprocess."""
 
     def _connect(self, argv):
-        proc = subprocess.Popen(argv,
-                                stdin=subprocess.PIPE,
-                                stdout=subprocess.PIPE,
+        # Attempt to make a socketpair to use as stdin/stdout for the SSH
+        # subprocess.  We prefer sockets to pipes because they support
+        # non-blocking short reads, allowing us to optimistically read 64k (or
+        # whatever) chunks.
+        try:
+            my_sock, subproc_sock = socket.socketpair()
+        except (AttributeError, socket.error):
+            # This platform doesn't support socketpair(), so just use ordinary
+            # pipes instead.
+            stdin = stdout = subprocess.PIPE
+            sock = None
+        else:
+            stdin = stdout = subproc_sock
+            sock = my_sock
+        proc = subprocess.Popen(argv, stdin=stdin, stdout=stdout,
                                 **os_specific_subprocess_params())
-        return SSHSubprocess(proc)
+        return SSHSubprocessConnection(proc, sock=sock)
 
     def connect_sftp(self, username, password, host, port):
         try:
@@ -652,11 +652,40 @@
             pass
 
 
-class SSHSubprocess(object):
-    """A socket-like object that talks to an ssh subprocess via pipes."""
-
-    def __init__(self, proc):
+class SSHConnection(object):
+    """Abstract base class for SSH connections."""
+
+    def get_sock_or_pipes(self):
+        """Returns a (kind, io_object) pair.
+
+        If kind == 'socket', then io_object is a socket.
+
+        If kind == 'pipes', then io_object is a pair of file-like objects
+        (read_from, write_to).
+        """
+        raise NotImplementedError(self.get_sock_or_pipes)
+
+    def close(self):
+        raise NotImplementedError(self.close)
+
+
+class SSHSubprocessConnection(SSHConnection):
+    """A connection to an ssh subprocess via pipes or a socket.
+
+    This class is also socket-like enough to be used with
+    SocketAsChannelAdapter (it has 'send' and 'recv' methods).
+    """
+
+    def __init__(self, proc, sock=None):
+        """Constructor.
+
+        :param proc: a subprocess.Popen
+        :param sock: if proc.stdin/out is a socket from a socketpair, then sock
+            should bzrlib's half of that socketpair.  If not passed, proc's
+            stdin/out is assumed to be ordinary pipes.
+        """
         self.proc = proc
+        self._sock = sock
         # Add a weakref to proc that will attempt to do the same as self.close
         # to avoid leaving processes lingering indefinitely.
         def terminate(ref):
@@ -665,14 +694,37 @@
         _subproc_weakrefs.add(weakref.ref(self, terminate))
 
     def send(self, data):
-        return os.write(self.proc.stdin.fileno(), data)
+        if self._sock is not None:
+            return self._sock.send(data)
+        else:
+            return os.write(self.proc.stdin.fileno(), data)
 
     def recv(self, count):
-        return os.read(self.proc.stdout.fileno(), count)
+        if self._sock is not None:
+            return self._sock.read(count)
+        else:
+            return os.read(self.proc.stdout.fileno(), count)
 
     def close(self):
         _close_ssh_proc(self.proc)
 
-    def get_filelike_channels(self):
-        return (self.proc.stdout, self.proc.stdin)
+    def get_sock_or_pipes(self):
+        if self._sock is not None:
+            return 'socket', self._sock
+        else:
+            return 'pipes', (self.proc.stdout, self.proc.stdin)
+
+
+class _ParamikoSSHConnection(SSHConnection):
+    """An SSH connection via paramiko."""
+
+    def __init__(self, channel):
+        self.channel = channel
+
+    def get_sock_or_pipes(self):
+        return ('socket', self.channel)
+
+    def close(self):
+        return self.channel.close()
+
 

=== modified file 'bzrlib/urlutils.py'
--- bzrlib/urlutils.py	2010-05-27 22:10:42 +0000
+++ bzrlib/urlutils.py	2010-06-15 04:32:49 +0000
@@ -101,7 +101,12 @@
     first_path_slash = path.find('/')
     if first_path_slash == -1:
         return len(scheme), None
-    return len(scheme), first_path_slash+len(scheme)+3
+    return len(scheme), first_path_slash+m.start('path')
+
+
+def is_url(url):
+    """Tests whether a URL is in actual fact a URL."""
+    return _url_scheme_re.match(url) is not None
 
 
 def join(base, *args):
@@ -118,67 +123,26 @@
     """
     if not args:
         return base
-    match = _url_scheme_re.match(base)
-    scheme = None
-    if match:
-        scheme = match.group('scheme')
-        path = match.group('path').split('/')
-        if path[-1:] == ['']:
-            # Strip off a trailing slash
-            # This helps both when we are at the root, and when
-            # 'base' has an extra slash at the end
-            path = path[:-1]
-    else:
-        path = base.split('/')
-
-    if scheme is not None and len(path) >= 1:
-        host = path[:1]
-        # the path should be represented as an abs path.
-        # we know this must be absolute because of the presence of a URL scheme.
-        remove_root = True
-        path = [''] + path[1:]
-    else:
-        # create an empty host, but dont alter the path - this might be a
-        # relative url fragment.
-        host = []
-        remove_root = False
-
+    scheme_end, path_start = _find_scheme_and_separator(base)
+    if scheme_end is None and path_start is None:
+        path_start = 0
+    elif path_start is None:
+        path_start = len(base)
+    path = base[path_start:]
     for arg in args:
-        match = _url_scheme_re.match(arg)
-        if match:
-            # Absolute URL
-            scheme = match.group('scheme')
-            # this skips .. normalisation, making http://host/../../..
-            # be rather strange.
-            path = match.group('path').split('/')
-            # set the host and path according to new absolute URL, discarding
-            # any previous values.
-            # XXX: duplicates mess from earlier in this function.  This URL
-            # manipulation code needs some cleaning up.
-            if scheme is not None and len(path) >= 1:
-                host = path[:1]
-                path = path[1:]
-                # url scheme implies absolute path.
-                path = [''] + path
-            else:
-                # no url scheme we take the path as is.
-                host = []
+        arg_scheme_end, arg_path_start = _find_scheme_and_separator(arg)
+        if arg_scheme_end is None and arg_path_start is None:
+            arg_path_start = 0
+        elif arg_path_start is None:
+            arg_path_start = len(arg)
+        if arg_scheme_end is not None:
+            base = arg
+            path = arg[arg_path_start:]
+            scheme_end = arg_scheme_end
+            path_start = arg_path_start
         else:
-            path = '/'.join(path)
             path = joinpath(path, arg)
-            path = path.split('/')
-    if remove_root and path[0:1] == ['']:
-        del path[0]
-    if host:
-        # Remove the leading slash from the path, so long as it isn't also the
-        # trailing slash, which we want to keep if present.
-        if path and path[0] == '' and len(path) > 1:
-            del path[0]
-        path = host + path
-
-    if scheme is None:
-        return '/'.join(path)
-    return scheme + '://' + '/'.join(path)
+    return base[:path_start] + path
 
 
 def joinpath(base, *args):
@@ -278,7 +242,7 @@
     # on non-win32 platform
     # FIXME: It turns out that on nt, ntpath.abspath uses nt._getfullpathname
     #       which actually strips trailing space characters.
-    #       The worst part is that under linux ntpath.abspath has different
+    #       The worst part is that on linux ntpath.abspath has different
     #       semantics, since 'nt' is not an available module.
     if path == '/':
         return 'file:///'
@@ -303,7 +267,7 @@
     MIN_ABS_FILEURL_LENGTH = WIN32_MIN_ABS_FILEURL_LENGTH
 
 
-_url_scheme_re = re.compile(r'^(?P<scheme>[^:/]{2,})://(?P<path>.*)$')
+_url_scheme_re = re.compile(r'^(?P<scheme>[^:/]{2,}):(//)?(?P<path>.*)$')
 _url_hex_escapes_re = re.compile(r'(%[0-9a-fA-F]{2})')
 
 
@@ -339,18 +303,18 @@
     :param url: Either a hybrid URL or a local path
     :return: A normalized URL which only includes 7-bit ASCII characters.
     """
-    m = _url_scheme_re.match(url)
-    if not m:
+    scheme_end, path_start = _find_scheme_and_separator(url)
+    if scheme_end is None:
         return local_path_to_url(url)
-    scheme = m.group('scheme')
-    path = m.group('path')
+    prefix = url[:path_start]
+    path = url[path_start:]
     if not isinstance(url, unicode):
         for c in url:
             if c not in _url_safe_characters:
                 raise errors.InvalidURL(url, 'URLs can only contain specific'
                                             ' safe characters (not %r)' % c)
         path = _url_hex_escapes_re.sub(_unescape_safe_chars, path)
-        return str(scheme + '://' + ''.join(path))
+        return str(prefix + ''.join(path))
 
     # We have a unicode (hybrid) url
     path_chars = list(path)
@@ -362,7 +326,7 @@
                 ['%%%02X' % ord(c) for c in path_chars[i].encode('utf-8')])
     path = ''.join(path_chars)
     path = _url_hex_escapes_re.sub(_unescape_safe_chars, path)
-    return str(scheme + '://' + path)
+    return str(prefix + path)
 
 
 def relative_url(base, other):

=== modified file 'bzrlib/win32utils.py'
--- bzrlib/win32utils.py	2010-05-20 02:57:52 +0000
+++ bzrlib/win32utils.py	2010-06-01 14:24:01 +0000
@@ -568,6 +568,10 @@
                 if argv[idx][:1] != '-':
                     break
             argv = argv[idx+1:]
+        # we should remove '--profile-imports' option as well (bug #588277)
+        # see bzr script ~ line 54
+        if '--profile-imports' in argv and '--profile-imports' not in sys.argv:
+            argv.remove('--profile-imports')
         return argv
 else:
     get_unicode_argv = None

=== modified file 'doc/developers/HACKING.txt'
--- doc/developers/HACKING.txt	2010-05-27 04:55:13 +0000
+++ doc/developers/HACKING.txt	2010-06-02 05:03:31 +0000
@@ -869,7 +869,7 @@
 energy by emailing the **bazaar-commits** list implicitly. To do this,
 install and configure the Email plugin. One way to do this is add these
 configuration settings to your central configuration file (e.g.
-``~/.bazaar/bazaar.conf`` on Linux)::
+``~/.bazaar/bazaar.conf``)::
 
   [DEFAULT]
   email = Joe Smith <joe.smith@internode.on.net>

=== modified file 'doc/developers/bug-handling.txt'
--- doc/developers/bug-handling.txt	2010-02-18 04:17:22 +0000
+++ doc/developers/bug-handling.txt	2010-05-31 19:33:05 +0000
@@ -303,6 +303,9 @@
 performance
     bugs about performance problems.
 
+regression
+    bugs which represent an aspect of bzr becoming accidentally less good than it was.
+
 test
     needs changes to the test framework
 

=== modified file 'doc/developers/case-insensitive-file-systems.txt'
--- doc/developers/case-insensitive-file-systems.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/case-insensitive-file-systems.txt	2010-06-02 05:03:31 +0000
@@ -9,13 +9,13 @@
 For example, the FAT32 file-system is most commonly found on Windows operating
 systems, and has the characteristics usually associated with a Windows
 file-system.  However, USB devices means FAT32 file-systems are often used
-with Linux, so the current operating system doesn't necessarily reflect the
+with GNU/Linux systems, so the current operating system doesn't necessarily reflect the
 capabilities of the file-system.
 
 Bazaar supports 3 kinds of file-systems, each to different degrees.
 
 * Case-sensitive file-systems: This is the file-system generally used on
-  Linux - 2 files can differ only by case, and the exact case must be used
+  GNU/Linux: 2 files can differ only by case, and the exact case must be used
   when opening a file.
 
 * Case-insensitive, case-preserving (cicp) file-systems: This is the

=== modified file 'doc/developers/code-style.txt'
--- doc/developers/code-style.txt	2010-05-27 04:55:13 +0000
+++ doc/developers/code-style.txt	2010-06-02 04:39:08 +0000
@@ -84,12 +84,12 @@
 
 Specifically:
 
- * Don't use the ``with`` statement.
-
- * Don't ``from . import``.
-
- * Don't use ``try/except/finally``, which is not supported in Python2.4,
-   use separate nested ``try/except`` and ``try/finally`` blocks.
+* Don't use the ``with`` statement.
+
+* Don't ``from . import``.
+
+* Don't use ``try/except/finally``, which is not supported in Python2.4,
+  use separate nested ``try/except`` and ``try/finally`` blocks.
 
 
 hasattr and getattr
@@ -152,32 +152,49 @@
 later time, or possibly never at all.  Therefore we have restrictions on
 what can be done inside them.
 
- 0. If you think you need to use a ``__del__`` method ask another
-    developer for alternatives.  If you do need to use one, explain
-    why in a comment.
-
- 1. Never rely on a ``__del__`` method running.  If there is code that
-    must run, do it from a ``finally`` block instead.
-
- 2. Never ``import`` from inside a ``__del__`` method, or you may crash the
-    interpreter!!
-
- 3. In some places we raise a warning from the destructor if the object
-    has not been cleaned up or closed.  This is considered OK: the warning
-    may not catch every case but it's still useful sometimes.
+0. If you think you need to use a ``__del__`` method ask another
+   developer for alternatives.  If you do need to use one, explain
+   why in a comment.
+
+1. Never rely on a ``__del__`` method running.  If there is code that
+   must run, do it from a ``finally`` block instead.
+
+2. Never ``import`` from inside a ``__del__`` method, or you may crash the
+   interpreter!!
+
+3. In some places we raise a warning from the destructor if the object
+   has not been cleaned up or closed.  This is considered OK: the warning
+   may not catch every case but it's still useful sometimes.
 
 
 Cleanup methods
 ===============
 
-Often when something has failed later code, including cleanups invoked
-from ``finally`` blocks, will fail too.  These secondary failures are
-generally uninteresting compared to the original exception.  So use the
-``only_raises`` decorator (from ``bzrlib.decorators``) for methods that
-are typically called in ``finally`` blocks, such as ``unlock`` methods.
-For example, ``@only_raises(LockNotHeld, LockBroken)``.  All errors that
-are unlikely to be a knock-on failure from a previous failure should be
-allowed.
+Often when something has failed later code will fail too, including
+cleanups invoked from ``finally`` blocks.  These secondary failures are
+generally uninteresting compared to the original exception.  ``bzrlib``
+has some facilities you can use to mitigate this.
+
+* In ``Command`` subclasses, prefer the ``add_cleanup`` method to using
+  ``try``/``finally`` blocks.  E.g. to acquire a lock and ensure it will
+  always be released when the command is done::
+
+    self.add_cleanup(branch.lock_read().unlock)
+
+  This also avoids heavily indented code. It also makes it easier to notice
+  mismatched lock/unlock pairs (and other kinds of resource
+  acquire/release) because there isn't a large block of code separating
+  them.
+
+* Use the ``only_raises`` decorator (from ``bzrlib.decorators``) when
+  defining methods that are typically called in ``finally`` blocks, such
+  as ``unlock`` methods.  For example, ``@only_raises(LockNotHeld,
+  LockBroken)``.  All errors that are unlikely to be a knock-on failure
+  from a previous failure should be allowed.
+
+* Consider using the ``OperationWithCleanups`` helper from
+  ``bzrlib.cleanup`` anywhere else you have a ``finally`` block that
+  might fail.
 
 
 Factories
@@ -354,25 +371,26 @@
 
 Rationale:
 
- * It makes the behaviour vary depending on whether bzr is run with -O
-   or not, therefore giving a chance for bugs that occur in one case or
-   the other, several of which have already occurred: assertions with
-   side effects, code which can't continue unless the assertion passes,
-   cases where we should give the user a proper message rather than an
-   assertion failure.
- * It's not that much shorter than an explicit if/raise.
- * It tends to lead to fuzzy thinking about whether the check is
-   actually needed or not, and whether it's an internal error or not
- * It tends to cause look-before-you-leap patterns.
- * It's unsafe if the check is needed to protect the integrity of the
-   user's data.
- * It tends to give poor messages since the developer can get by with
-   no explanatory text at all.
- * We can't rely on people always running with -O in normal use, so we
-   can't use it for tests that are actually expensive.
- * Expensive checks that help developers are better turned on from the
-   test suite or a -D flag.
- * If used instead of ``self.assert*()`` in tests it makes them falsely pass with -O.
+* It makes the behaviour vary depending on whether bzr is run with -O
+  or not, therefore giving a chance for bugs that occur in one case or
+  the other, several of which have already occurred: assertions with
+  side effects, code which can't continue unless the assertion passes,
+  cases where we should give the user a proper message rather than an
+  assertion failure.
+* It's not that much shorter than an explicit if/raise.
+* It tends to lead to fuzzy thinking about whether the check is
+  actually needed or not, and whether it's an internal error or not
+* It tends to cause look-before-you-leap patterns.
+* It's unsafe if the check is needed to protect the integrity of the
+  user's data.
+* It tends to give poor messages since the developer can get by with
+  no explanatory text at all.
+* We can't rely on people always running with -O in normal use, so we
+  can't use it for tests that are actually expensive.
+* Expensive checks that help developers are better turned on from the
+  test suite or a -D flag.
+* If used instead of ``self.assert*()`` in tests it makes them falsely
+  pass with -O.
 
 emacs setup
 ===========
@@ -423,5 +441,17 @@
     finally:
         f.close()
 
+
+Terminology
+===========
+
+Bazaar is a GNU project and uses standard GNU terminology, especially:
+
+ * Use the word "Linux" to refer to the Linux kernel, not as a synechoche
+   for the entire operating system.  (See `bug 528253
+   <https://bugs.launchpad.net/bzr/+bug/528253>`_).
+
+ * Don't say "open source" when you mean "free software".
+
 ..
    vim: ft=rst tw=74 ai

=== modified file 'doc/developers/planned-change-integration.txt'
--- doc/developers/planned-change-integration.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/planned-change-integration.txt	2010-06-02 05:03:31 +0000
@@ -73,7 +73,7 @@
  * Working tree disk ordering: Knowing the expected order for disk operations
    may influence the needed use case specific APIs, so having a solid
    understanding of what is optimal - and why - and whether it is pessimal on
-   non linux platforms is rather important.
+   non-Linux-kernel platforms is rather important.
 
  * Be able to version files greater than memory in size: This cannot be
    achieved until all parts of the library which deal with user files are able

=== modified file 'doc/developers/testing.txt'
--- doc/developers/testing.txt	2010-05-12 10:51:28 +0000
+++ doc/developers/testing.txt	2010-06-11 06:19:48 +0000
@@ -395,20 +395,21 @@
 The actual use of ScriptRunner within a TestCase looks something like
 this::
 
-        def test_unshelve_keep(self):
-                # some setup here
-                sr = ScriptRunner()
-                sr.run_script(self, '''
-        $ bzr add file
-        $ bzr shelve --all -m Foo
-        $ bzr shelve --list
-        1: Foo
-        $ bzr unshelve --keep
-        $ bzr shelve --list
-        1: Foo
-        $ cat file
-        contents of file
-        ''')
+    from bzrlib.tests import script
+
+    def test_unshelve_keep(self):
+        # some setup here
+        script.run_script(self, '''
+            $ bzr add file
+            $ bzr shelve --all -m Foo
+            $ bzr shelve --list
+            1: Foo
+            $ bzr unshelve --keep
+            $ bzr shelve --list
+            1: Foo
+            $ cat file
+            contents of file
+            ''')
 
 
 Import tariff tests
@@ -430,7 +431,7 @@
   dependencies and only needed for particular cases.  If they're loaded in
   other cases then bzr may break for people who don't have those modules.
   
-`test_import_tarrif` allows us to check that removal of imports doesn't
+`test_import_tariff` allows us to check that removal of imports doesn't
 regress.
 
 This is done by running the command in a subprocess with
@@ -872,4 +873,4 @@
 .. |--| unicode:: U+2014
 
 ..
-   vim: ft=rst tw=74 ai
+   vim: ft=rst tw=74 ai et sw=4

=== modified file 'doc/en/admin-guide/code-browsing.txt'
--- doc/en/admin-guide/code-browsing.txt	2009-12-11 00:00:04 +0000
+++ doc/en/admin-guide/code-browsing.txt	2010-06-02 05:03:31 +0000
@@ -47,8 +47,8 @@
    On Ubuntu, `sudo apt-get install python-flup`
    or use `easy_install flup`
 
-Although directions for installing these on Ubuntu Linux are given, most other
-Linux distributions should package these dependencies, making installation
+Although directions for installing these on Ubuntu are given, most other
+GNU/Linux distributions should package these dependencies, making installation
 easy.  For Windows and Mac OS X, they should all be ``easy_install``-able or at
 worst installable from the Python sources.
 
@@ -101,9 +101,9 @@
 This would allow the trunk branch of ProjectX to be browsed at
 ``http://www.example.com/loggerhead/projectx/trunk``.
 
-Loggerhead comes with a script allowing it to run as a service on init.d based
-Linux systems.  Contributions to do a similar thing on Windows servers would
-be welcomed at http://launchpad.net/loggerhead.
+Loggerhead comes with a script allowing it to run as a service on
+``init.d`` based Unix systems.  Contributions to do a similar thing on
+Windows servers would be welcomed at http://launchpad.net/loggerhead.
   
 
 Other web interfaces

=== modified file 'doc/en/admin-guide/introduction.txt'
--- doc/en/admin-guide/introduction.txt	2009-12-07 21:51:01 +0000
+++ doc/en/admin-guide/introduction.txt	2010-06-02 05:03:31 +0000
@@ -27,7 +27,7 @@
 What you need to run a Bazaar server
 ------------------------------------
 
-Where possible, we will discuss both Unix (including Linux) and Windows server
+Where possible, we will discuss both Unix (including GNU/Linux) and Windows server
 environments.  For the purposes of this document, we will consider Mac OS X as
 a type of Unix.
 
@@ -44,7 +44,7 @@
 critical components of the code.  Pure Python alternatives exist for all of
 these components, but they may be considerably slower.  To compile these
 extensions, you need a C compiler and the relevant header files from the
-Python package.  On Linux, these may be in a separate package.  Other
+Python package.  On GNU/Linux, these may be in a separate package.  Other
 operating systems should have the required headers installed by default.
 
 If you are installing a development version of Bazaar, rather than a released

=== modified file 'doc/en/admin-guide/upgrade.txt'
--- doc/en/admin-guide/upgrade.txt	2009-12-18 10:09:49 +0000
+++ doc/en/admin-guide/upgrade.txt	2010-06-02 05:03:31 +0000
@@ -28,7 +28,7 @@
 
 Upgrading the Bazaar software is as simple as re-installing the Python package
 using either the latest binary package for Windows or Mac OS X, the binary
-package provided by your Linux distribution, or installing from the source
+package provided by your GNU/Linux distribution, or installing from the source
 release.  See http://bazaar-vcs.org/Downloads for the latest releases for all
 supported platforms.
 

=== modified file 'doc/en/tutorials/using_bazaar_with_launchpad.txt'
--- doc/en/tutorials/using_bazaar_with_launchpad.txt	2009-12-02 20:34:07 +0000
+++ doc/en/tutorials/using_bazaar_with_launchpad.txt	2010-06-02 05:18:27 +0000
@@ -23,7 +23,7 @@
 the software: the team, users, training partners, support partners,
 third-party developers and so on.
 
-Great communities are well understood in the open source world.
+Great communities are well understood in the free software community.
 Their applicability extends well beyond that though: most
 successful commercial software vendors are well skilled at
 building and managing the communities that grow up around
@@ -93,8 +93,8 @@
 
 As well as sponsoring `Ubuntu <http://www.ubuntu.com>`_ and
 `Bazaar <http://bazaar-vcs.org>`_ development, Canonical
-provides Launchpad, https://launchpad.net, as a free service
-for the open source community. Launchpad is one of the most
+provides Launchpad, <https://launchpad.net/>, as a free service
+for the community. Launchpad is one of the most
 exciting CDEs around for several notable reasons:
 
 * it models relationships between many of things tracked, e.g.
@@ -179,10 +179,10 @@
 Accessing code in Launchpad using Bazaar
 ========================================
 
-Getting the code for an open source project
--------------------------------------------
+Getting the code for a project
+------------------------------
 
-As Launchpad keeps track of thousands of open source projects
+As Launchpad keeps track of thousands of projects
 and their latest code whether it be managed by Bazaar, CVS or Subversion,
 Bazaar users can grab that code as easily as this::
 
@@ -447,8 +447,7 @@
 further integrated, please contact us on the Bazaar mailing list,
 bazaar@lists.canonical.com.
 
-While designed as a free service to support open source projects,
+While designed as a free service to support free software projects,
 Canonical may make Launchpad available to commercial software developers
 depending on their requirements. We would be happy to hear from you
-if you think Launchpad would be useful for managing your community,
-open source or otherwise.
+if you think Launchpad would be useful for managing your community.

=== modified file 'doc/en/upgrade-guide/overview.txt'
--- doc/en/upgrade-guide/overview.txt	2009-07-13 06:58:49 +0000
+++ doc/en/upgrade-guide/overview.txt	2010-06-02 05:03:31 +0000
@@ -36,7 +36,7 @@
 to Bazaar 1.y. In either case, a brief outline of the steps is given
 below.
 
-To upgrade Bazaar on Linux:
+To upgrade Bazaar on Ubuntu:
 
 1. Ensure your package manager is configured with the required
    software sources, e.g. the official release PPA for Ubuntu:
@@ -72,10 +72,10 @@
 bzr-svn, are more tightly associated with Bazaar's APIs so these
 typically need to be upgraded in lockstep with the core software.
 
-For Windows and OS X users, bzrtools and bzr-svn are typically
-included in the installer so no special steps are required to upgrade
-these. For Linux and UNIX users, bztrools, bzr-svn and many other
-popular plugins can be installed and upgraded using your
+For Windows and OS X users, bzrtools and bzr-svn are typically included in
+the installer so no special steps are required to upgrade these. For
+Ubuntu and other GNU/Linux or Unix systems users, bztrools, bzr-svn and
+many other popular plugins can be installed and upgraded using your
 platform's package manager, e.g. Synaptic on Ubuntu.
 
 

=== modified file 'doc/en/user-guide/branching_a_project.txt'
--- doc/en/user-guide/branching_a_project.txt	2010-03-22 00:29:50 +0000
+++ doc/en/user-guide/branching_a_project.txt	2010-06-02 05:03:31 +0000
@@ -8,7 +8,7 @@
 agree on a transfer technology.
 You may decide to make the top level directory of your branch
 a network share, an approach familiar to Windows users.
-Linux and OS X users might prefer access to be
+Unix users might prefer access to be
 via SFTP, a secure protocol built-in to most SSH servers.
 Bazaar is *very* flexible in this regard with support for
 lots of protocols some of which are given below.

=== modified file 'doc/en/user-guide/configuring_bazaar.txt'
--- doc/en/user-guide/configuring_bazaar.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/configuring_bazaar.txt	2010-06-02 05:03:31 +0000
@@ -19,7 +19,7 @@
 Configuration files
 -------------------
 
-Configuration files are located in ``$HOME/.bazaar`` on Linux/Unix and
+Configuration files are located in ``$HOME/.bazaar`` on Unix and
 ``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0`` on
 Windows. There are three primary configuration files in this location:
 

=== modified file 'doc/en/user-guide/installing_bazaar.txt'
--- doc/en/user-guide/installing_bazaar.txt	2009-09-09 15:08:17 +0000
+++ doc/en/user-guide/installing_bazaar.txt	2010-06-02 05:03:31 +0000
@@ -1,11 +1,11 @@
 Installing Bazaar
 =================
 
-Linux
------
+GNU/Linux
+---------
 
-Bazaar packages are available for most popular Linux distributions
-including Ubuntu/Debian, Red Hat and Gentoo.
+Bazaar packages are available for most popular GNU/Linux distributions
+including Ubuntu, Debian, Red Hat and Gentoo.
 See http://bazaar-vcs.org/Download for the latest instructions.
 
 Windows

=== modified file 'doc/en/user-guide/introducing_bazaar.txt'
--- doc/en/user-guide/introducing_bazaar.txt	2010-03-11 04:33:41 +0000
+++ doc/en/user-guide/introducing_bazaar.txt	2010-06-02 05:18:27 +0000
@@ -79,7 +79,7 @@
  * trunk quality and stability can be kept higher, making everyone's
    job less stressful
 
- * in open source communities:
+ * in open communities:
 
    * easier for non-core developers to create and maintain changes
 

=== modified file 'doc/en/user-guide/plugins.txt'
--- doc/en/user-guide/plugins.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/plugins.txt	2010-06-02 05:03:31 +0000
@@ -35,13 +35,13 @@
 
 Installing a plugin is very easy! If not already created, create a
 ``plugins`` directory under your Bazaar configuration directory,
-``~/.bazaar/`` on Linux and
+``~/.bazaar/`` on Unix and
 ``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0\``
 on Windows. Within this directory (referred to as $BZR_HOME below),
 each plugin is placed in its own subdirectory.
 
 Plugins work particularly well with Bazaar branches. For example, to
-install the bzrtools plugins for your main user account on Linux,
+install the bzrtools plugins for your main user account on GNU/Linux,
 one can perform the following::
 
     bzr branch http://panoramicfeedback.com/opensource/bzr/bzrtools

=== modified file 'doc/en/user-guide/setting_up_email.txt'
--- doc/en/user-guide/setting_up_email.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/setting_up_email.txt	2010-06-02 05:03:31 +0000
@@ -62,7 +62,7 @@
 --------------------------------------------
 
 To use the default ini file, create or edit the ``bazaar.conf`` file (in
-``~/.bazaar/`` on Linux and in ``%APPDATA%\bazaar\2.0\`` in Windows)
+``~/.bazaar/`` on Unix and in ``%APPDATA%\bazaar\2.0\`` in Windows)
 and set an email address as shown below.  Please note that the word DEFAULT
 is case sensitive, and must be in upper-case.
 ::

=== modified file 'doc/en/user-guide/version_info.txt'
--- doc/en/user-guide/version_info.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/version_info.txt	2010-06-02 05:03:31 +0000
@@ -25,7 +25,7 @@
   branch-nick: bzr.dev
 
 You can easily filter that output using operating system tools or
-scripts. For example (on Linux/Unix)::
+scripts. For example::
 
   $ bzr version-info | grep ^date
   date: 2007-12-11 17:51:18 +0000

=== modified file 'tools/check-newsbugs.py'
--- tools/check-newsbugs.py	2009-01-30 16:55:33 +0000
+++ tools/check-newsbugs.py	2010-06-01 16:46:47 +0000
@@ -4,9 +4,9 @@
 
 import getopt, re, sys
 try:
-    from launchpadbugs import connector
+    from launchpadlib.launchpad import Launchpad
 except ImportError:
-    print "Please install launchpadbugs from lp:python-launchpad-bugs"
+    print "Please install launchpadlib from lp:launchpadlib"
     sys.exit(1)
 
 options, args = getopt.gnu_getopt(sys.argv, "l", ["launchpad"])
@@ -22,13 +22,13 @@
 
 def report_notmarked(bug, task, section):
     print 
-    print "Bug %d was mentioned in NEWS but is not marked fix released:" % (bug.bugnumber, )
+    print "Bug %d was mentioned in NEWS but is not marked fix released:" % (bug.id, )
     print "Launchpad title: %s" % bug.title
     print "NEWS summary: "
     print section
     if "--launchpad" in options or "-l" in options:
-        print "  bug %d" % bug.bugnumber
-        print "  affects bzr"
+        print "  bug %d" % bug.id
+        print "  affects %s" % task.bug_target_name
         print "  status fixreleased"
 
 
@@ -60,14 +60,15 @@
     finally:
         f.close()
 
-open_bug = connector.ConnectBug("TEXT")
+
+lp = Launchpad.login_anonymously('bzr-check-newsbugs', 'edge', version='1.0')
 
 bugnos = read_news_bugnos(args[1])
 for bugno, section in bugnos:
-    bug = open_bug(url="https://bugs.launchpad.net/bzr/+bug/%d" % bugno)
+    bug = lp.bugs[bugno]
     found_bzr = False
-    for task in bug.infotable:
-        if task.affects == "bzr":
+    for task in bug.bug_tasks:
+        if task.bug_target_name == "bzr":
             found_bzr = True
             if task.status != "Fix Released":
                 report_notmarked(bug, task, section)

